@@ -1,10 +1,9 @@
-from threading import Condition
-
 from gi.repository import GObject
-from pychess.System.Log import log
+
 from pychess.Players.Engine import Engine
 from pychess.Utils.const import NORMAL
-from pychess.Utils.repr import reprColor
+
+TIME_OUT_SECOND = 10
 
 
 class ProtocolEngine(Engine):
@@ -29,7 +28,3 @@ class ProtocolEngine(Engine):
 
         self.connected = True
         self.mode = NORMAL
-
-        log.debug(reprColor[color], extra={"task": self.defname})
-
-        self.movecon = Condition()
