@@ -1,28 +1,27 @@
-from __future__ import absolute_import
-from __future__ import print_function
+import asyncio
 import os
 from os.path import join, dirname, abspath
 import sys
+import shutil
 import json
 import platform
 from functools import partial
 from hashlib import md5
-from threading import Thread, Lock
 from copy import deepcopy
+from collections import OrderedDict
 
-from gi.repository import GObject
+from gi.repository import GLib, GObject
 
-from pychess.System import conf, fident
+from pychess.System import conf
 from pychess.System.Log import log
 from pychess.System.command import Command
-from pychess.System.SubProcess import SubProcess, searchPath, SubProcessError
+from pychess.System.SubProcess import SubProcess
 from pychess.System.prefix import addUserConfigPrefix, getDataPrefix, getEngineDataPrefix
 from pychess.Players.Player import PlayerIsDead
 from pychess.Utils import createStoryTextAppEvent
-from pychess.Utils.const import BLACK, UNKNOWN_REASON, SUBPROCESS_SUBPROCESS, WHITE, \
-    HINT, ANALYZING, INVERSE_ANALYZING
-from .CECPEngine import CECPEngine
-from .UCIEngine import UCIEngine
+from pychess.Utils.const import BLACK, UNKNOWN_REASON, WHITE, HINT, ANALYZING, INVERSE_ANALYZING, NORMALCHESS
+from pychess.Players.CECPEngine import CECPEngine
+from pychess.Players.UCIEngine import UCIEngine
 from pychess.Variants import variants
 
 attrToProtocol = {"uci": UCIEngine, "xboard": CECPEngine}
@@ -33,7 +32,7 @@ BITNESS = "64" if platform.machine().endswith('64') else "32"
 if sys.platform == "win32":
     backup = [
         {"protocol": "uci",
-         "name": "stockfish-7-%s.exe" % BITNESS,
+         "name": "stockfish_8_x%s.exe" % BITNESS,
          "country": "no"},
         {"protocol": "xboard",
          "name": "sjaakii_win%s_ms.exe" % BITNESS,
@@ -135,6 +134,10 @@ else:
     ]
 
 
+class SubProcessError(Exception):
+    pass
+
+
 def md5_sum(filename):
     with open(filename, mode='rb') as file_handle:
         md5sum = md5()
@@ -182,70 +185,71 @@ class EngineDiscoverer(GObject.GObject):
                         ext = os.path.splitext(entry)[1]
                         if ext == ".eng":
                             with open(os.path.join(path, entry)) as file_handle:
+                                plugin_spec = file_handle.readline().strip()
+                                if not plugin_spec.startswith("plugin spec"):
+                                    continue
+
                                 engine_command = file_handle.readline().strip()
+
+                                supported_variants = file_handle.readline().strip()
+                                if not supported_variants.startswith("chess"):
+                                    continue
+
                                 new_engine = {}
-                                if engine_command.startswith(
-                                        "cd ") and engine_command.find(
-                                            ";") > 0:
+                                if engine_command.startswith("cd ") and engine_command.find(";") > 0:
                                     parts = engine_command.split(";")
                                     working_directory = parts[0][3:]
                                     engine_command = parts[1]
-                                    new_engine[
-                                        "workingDirectory"] = working_directory
+                                    new_engine["workingDirectory"] = working_directory
 
                                 find = False
                                 for engine in self._engines:
                                     if engine["name"] == engine_command:
                                         find = True
                                         break
+
                                 if not find:
                                     new_engine["protocol"] = protocol
                                     new_engine["name"] = engine_command
                                     self._engines.append(new_engine)
 
-                                    ############################################################################
-                                    # Discover methods                                                         #
-                                    ############################################################################
-
     def __findRundata(self, engine):
         """ Searches for a readable, executable named 'name' in the PATH.
             For the PyChess engine, special handling is taken, and we search
             PYTHONPATH as well as the directory from where the 'os' module is
             imported """
         if engine.get("vm_name") is not None:
-            altpath = engine.get("vm_command")
+            vm_command = engine.get("vm_command")
+            altpath = dirname(vm_command) if vm_command else None
             if getattr(sys, 'frozen', False) and engine["vm_name"] == "wine":
                 vmpath = None
             else:
-                vmpath = searchPath(engine["vm_name"],
-                                    access=os.R_OK | os.X_OK,
-                                    altpath=altpath)
+                vmpath = shutil.which(engine["vm_name"], mode=os.R_OK | os.X_OK, path=altpath)
 
-            if engine["name"] == "PyChess.py" and not getattr(sys, 'frozen',
-                                                              False):
+            if engine["name"] == "PyChess.py" and not getattr(sys, 'frozen', False):
                 path = join(abspath(dirname(__file__)), "PyChess.py")
+                if not vmpath.endswith(PYTHONBIN):
+                    # from python to python3
+                    engine["vm_name"] = PYTHONBIN
+                    vmpath = shutil.which(PYTHONBIN, mode=os.R_OK | os.X_OK, path=None)
                 if not os.access(path, os.R_OK):
                     path = None
             else:
-                altpath = engine.get("command")
-                path = searchPath(engine["name"],
-                                  access=os.R_OK,
-                                  altpath=altpath)
+                command = engine.get("command")
+                altpath = dirname(command) if command else None
+                path = shutil.which(engine["name"], mode=os.R_OK, path=altpath)
 
             if vmpath and path:
                 return vmpath, path
-            elif path and sys.platform == "win32" and engine.get(
-                    "vm_name") == "wine":
+            elif path and sys.platform == "win32" and engine.get("vm_name") == "wine":
                 return None, path
 
         else:
-            altpath = engine.get("command")
+            command = engine.get("command")
+            altpath = dirname(command) if command else None
             if sys.platform == "win32" and not altpath:
-                altpath = os.path.join(getDataPrefix(), "engines",
-                                       engine["name"])
-            path = searchPath(engine["name"],
-                              access=os.R_OK | os.X_OK,
-                              altpath=altpath)
+                altpath = os.path.join(getDataPrefix(), "engines") + ";" + os.path.dirname(sys.executable)
+            path = shutil.which(engine["name"], mode=os.R_OK | os.X_OK, path=altpath)
             if path:
                 return None, path
         return False
@@ -273,21 +277,26 @@ class EngineDiscoverer(GObject.GObject):
 
         return engine
 
+    @asyncio.coroutine
     def __discoverE(self, engine):
-        subproc = self.initEngine(engine, BLACK)
         try:
+            subproc = yield from self.initEngine(engine, BLACK)
             subproc.connect('readyForOptions', self.__discoverE2, engine)
             subproc.prestart()  # Sends the 'start line'
-            subproc.start()
+
+            event = asyncio.Event()
+            subproc.start(event)
+            yield from event.wait()
         except SubProcessError as err:
             log.warning("Engine %s failed discovery: %s" % (engine["name"], err))
             self.emit("engine_failed", engine["name"], engine)
+            subproc.kill(UNKNOWN_REASON)
         except PlayerIsDead as err:
             # Check if the player died after engine_discovered by our own hands
             if not self.toBeRechecked[engine["name"]][1]:
-                log.warning("Engine %s failed discovery: %s" %
-                            (engine["name"], err))
+                log.warning("Engine %s failed discovery: %s" % (engine["name"], err))
                 self.emit("engine_failed", engine["name"], engine)
+            subproc.kill(UNKNOWN_REASON)
 
     def __discoverE2(self, subproc, engine):
         if engine.get("protocol") == "uci":
@@ -295,7 +304,6 @@ class EngineDiscoverer(GObject.GObject):
         elif engine.get("protocol") == "xboard":
             fresh = self.__fromCECPProcess(subproc)
         engine.update(fresh)
-
         exitcode = subproc.kill(UNKNOWN_REASON)
         if exitcode:
             log.debug("Engine failed %s" % engine["name"])
@@ -351,7 +359,7 @@ class EngineDiscoverer(GObject.GObject):
             engine["country"] = backup_engine["country"]
         except StopIteration:
             log.warning(
-                "Engine '%s' has not been tested and verified to work with PyChess" % engine.get('name'))
+                "Engine '%s' is not in PyChess predefined known engines list" % engine.get('name'))
             engine['recheck'] = True
 
         ######
@@ -377,20 +385,7 @@ class EngineDiscoverer(GObject.GObject):
             log.error("Saving engines.json raised exception: %s" %
                       ", ".join(str(a) for a in err.args))
 
-    def discover(self):
-        class Discoverer(Thread):
-            def __init__(self, parent):
-                Thread.__init__(self, name=fident(self.run))
-                self.daemon = True
-                self.parent = parent
-
-            def run(self):
-                self.parent.do_discover()
-
-        dis_coverer = Discoverer(self)
-        dis_coverer.start()
-
-    def do_discover(self):
+    def pre_discover(self):
         self.engines = []
         # List available engines
         for engine in self._engines:
@@ -410,18 +405,20 @@ class EngineDiscoverer(GObject.GObject):
         ######
         # Runs all the engines in toBeRechecked, in order to gather information
         ######
-        self.toBeRechecked = dict((c["name"], [c, False])
-                                  for c in self._engines if c.get('recheck'))
-        self.all_done_lock = Lock()
+        self.toBeRechecked = sorted([(c["name"], [c, False])
+                                    for c in self._engines if c.get('recheck')])
+        self.toBeRechecked = OrderedDict(self.toBeRechecked)
+
+    def discover(self):
+        self.pre_discover()
 
         def count(self_, name, engine, wentwell):
-            with self.all_done_lock:
-                if wentwell:
-                    self.toBeRechecked[name][1] = True
-                if all([elem[1] for elem in self.toBeRechecked.values()]):
-                    self.engines.sort(key=lambda x: x["name"])
-                    self.emit("all_engines_discovered")
-                    createStoryTextAppEvent("all_engines_discovered")
+            if wentwell:
+                self.toBeRechecked[name][1] = True
+            if all([elem[1] for elem in self.toBeRechecked.values()]):
+                self.engines.sort(key=lambda x: x["name"])
+                self.emit("all_engines_discovered")
+                createStoryTextAppEvent("all_engines_discovered")
 
         self.connect("engine_discovered", count, True)
         self.connect("engine_failed", count, False)
@@ -430,7 +427,7 @@ class EngineDiscoverer(GObject.GObject):
             self.connect("all_engines_discovered", self.save)
             for engine, done in self.toBeRechecked.values():
                 if not done:
-                    self.__discoverE(engine)
+                    asyncio.async(self.__discoverE(engine))
         else:
             self.emit("all_engines_discovered")
             createStoryTextAppEvent("all_engines_discovered")
@@ -472,33 +469,41 @@ class EngineDiscoverer(GObject.GObject):
                 return engine
 
     def getEngineVariants(self, engine):
+        UCI_without_standard_variant = False
+        engine_variants = []
         for variantClass in variants.values():
             if variantClass.standard_rules:
-                yield variantClass.variant
+                engine_variants.append(variantClass.variant)
             else:
                 if engine.get("variants"):
                     if variantClass.cecp_name in engine.get("variants"):
-                        yield variantClass.variant
+                        engine_variants.append(variantClass.variant)
                 # UCI knows Chess960 only
                 if engine.get("options"):
                     for option in engine["options"]:
                         if option["name"] == "UCI_Chess960" and \
                                 variantClass.cecp_name == "fischerandom":
-                            yield variantClass.variant
+                            engine_variants.append(variantClass.variant)
                         elif option["name"] == "UCI_Variant":
+                            UCI_without_standard_variant = "chess" not in option["choices"]
                             if variantClass.cecp_name in option["choices"] or \
                                     variantClass.cecp_name.lower().replace("-", "") in option["choices"]:
-                                yield variantClass.variant
+                                engine_variants.append(variantClass.variant)
+        # Fix #1444 Unsupported variant on the home screen
+        if UCI_without_standard_variant:
+            # this fix causes problems in TaskerManager and newGameDialog
+            # TODO: create correct fix
+            if 0:
+                engine_variants.remove(NORMALCHESS)
+        return engine_variants
 
     def getName(self, engine=None):
-        # Test if the call was to get the name of the thread
-        if engine is None:
-            return Thread.getName(self)
         return engine["name"]
 
     def getCountry(self, engine):
         return engine.get("country")
 
+    @asyncio.coroutine
     def initEngine(self, engine, color):
         name = engine['name']
         protocol = engine["protocol"]
@@ -522,15 +527,25 @@ class EngineDiscoverer(GObject.GObject):
         else:
             workdir = getEngineDataPrefix()
         warnwords = ("illegal", "error", "exception")
-        subprocess = SubProcess(path, args, warnwords, SUBPROCESS_SUBPROCESS,
-                                workdir)
+        try:
+            subprocess = SubProcess(path, args=args, warnwords=warnwords, cwd=workdir)
+            yield from subprocess.start()
+        except OSError:
+            raise PlayerIsDead
+        except asyncio.TimeoutError:
+            raise PlayerIsDead
+        except GLib.GError:
+            raise PlayerIsDead
+        except:
+            raise PlayerIsDead
+
         engine_proc = attrToProtocol[protocol](subprocess, color, protover,
                                                md5_engine)
-
         engine_proc.setName(name)
 
         # If the user has configured special options for this engine, here is
         # where they should be set.
+
         def optionsCallback(set_option):
             if engine.get("options"):
                 for option in engine["options"]:
@@ -545,6 +560,7 @@ class EngineDiscoverer(GObject.GObject):
 
         return engine_proc
 
+    @asyncio.coroutine
     def initPlayerEngine(self,
                          engine,
                          color,
@@ -552,21 +568,23 @@ class EngineDiscoverer(GObject.GObject):
                          variant,
                          secs=0,
                          incr=0,
+                         moves=0,
                          forcePonderOff=False):
-        engine = self.initEngine(engine, color)
+        engine = yield from self.initEngine(engine, color)
 
         def optionsCallback(engine):
             engine.setOptionStrength(diffi, forcePonderOff)
             engine.setOptionVariant(variant)
             if secs > 0:
-                engine.setOptionTime(secs, incr)
+                engine.setOptionTime(secs, incr, moves)
 
         engine.connect("readyForOptions", optionsCallback)
         engine.prestart()
         return engine
 
+    @asyncio.coroutine
     def initAnalyzerEngine(self, engine, mode, variant):
-        engine = self.initEngine(engine, WHITE)
+        engine = yield from self.initEngine(engine, WHITE)
 
         def optionsCallback(engine):
             engine.setOptionAnalyzing(mode)
@@ -576,13 +594,15 @@ class EngineDiscoverer(GObject.GObject):
         engine.prestart()
         return engine
 
-    def addEngine(self, name, new_engine, protocol, vm_name):
+    def addEngine(self, name, new_engine, protocol, vm_name, vm_args):
         engine = {"name": name,
                   "protocol": protocol,
                   "command": new_engine,
                   "recheck": True}
         if vm_name is not None:
             engine["vm_name"] = vm_name
+        if vm_args is not None:
+            engine["vm_args"] = vm_args
         self._engines.append(engine)
 
     def removeEngine(self, name):
@@ -598,6 +618,7 @@ class EngineDiscoverer(GObject.GObject):
 discoverer = EngineDiscoverer()
 
 
+@asyncio.coroutine
 def init_engine(analyzer_type, gamemodel, force=False):
     """
     Initializes and starts the engine analyzer of analyzer_type the user has
@@ -625,16 +646,16 @@ def init_engine(analyzer_type, gamemodel, force=False):
             engine = anaengines[-1]
 
         if gamemodel.variant.variant in discoverer.getEngineVariants(engine):
-            analyzer = discoverer.initAnalyzerEngine(engine, mode,
-                                                     gamemodel.variant)
+            analyzer = yield from discoverer.initAnalyzerEngine(engine, mode,
+                                                                gamemodel.variant)
             log.debug("%s analyzer: %s" % (analyzer_type, repr(analyzer)))
 
     return analyzer
 
 
 def is_uci(engine_command):
-    command = Command(engine_command, "uci\n")
-    status, output, err = command.run(timeout=3)
+    command = Command(engine_command, "uci\nquit\n")
+    status, output, err = command.run(timeout=5)
     uci = False
     for line in output.splitlines():
         line = line.rstrip()
@@ -647,8 +668,8 @@ def is_uci(engine_command):
 
 
 def is_cecp(engine_command):
-    command = Command(engine_command, "xboard\nprotover 2\n")
-    status, output, err = command.run(timeout=3)
+    command = Command(engine_command, "xboard\nprotover 2\nquit\n")
+    status, output, err = command.run(timeout=5)
     cecp = False
     for line in output.splitlines():
         line = line.rstrip()
@@ -661,12 +682,9 @@ def is_cecp(engine_command):
 
 
 if __name__ == "__main__":
-    from gi.repository import GLib
-    from gi.repository import GObject
-    GObject.threads_init()
-    mainloop = GLib.MainLoop()
-
-    # discoverer = EngineDiscoverer()
+    from pychess.external import gbulb
+    gbulb.install()
+    loop = asyncio.get_event_loop()
 
     def discovering_started(discoverer, names):
         print("discovering_started", names)
@@ -681,10 +699,10 @@ if __name__ == "__main__":
     def all_engines_discovered(discoverer):
         print("all_engines_discovered")
         print([engine["name"] for engine in discoverer.getEngines()])
-        mainloop.quit()
+        loop.stop()
 
     discoverer.connect("all_engines_discovered", all_engines_discovered)
 
-    discoverer.start()
+    discoverer.discover()
 
-    mainloop.run()
+    loop.run_forever()
