@@ -1,20 +1,18 @@
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
+import collections
 
-from pychess.compat import strip
 from .ChessFile import ChessFile, LoadingError
 from pychess.Utils.GameModel import GameModel
-from pychess.Utils.const import WHITE, BLACK, WON_RESIGN, WAITING_TO_START, BLACKWON, WHITEWON, DRAW
+from pychess.Utils.const import WHITE, BLACK, WON_RESIGN, WAITING_TO_START, BLACKWON, WHITEWON, DRAW, FISCHERRANDOMCHESS
 from pychess.Utils.logic import getStatus
 from pychess.Utils.lutils.leval import evaluateComplete
+from pychess.Variants.fischerandom import FischerandomBoard
 
 __label__ = _("Chess Position")
 __ending__ = "epd"
 __append__ = True
 
 
-def save(file, model, position=None):
+def save(handle, model, position=None):
     """Saves game to file in fen format"""
 
     color = model.boards[-1].color
@@ -22,7 +20,7 @@ def save(file, model, position=None):
     fen = model.boards[-1].asFen().split(" ")
 
     # First four parts of fen are the same in epd
-    file.write(" ".join(fen[:4]))
+    handle.write(" ".join(fen[:4]))
 
     ############################################################################
     # Repetition count                                                         #
@@ -61,30 +59,53 @@ def save(file, model, position=None):
         ("rc", rep_count), )
 
     for key, value in opcodes:
-        file.write(" %s %s;" % (key, value))
+        handle.write(" %s %s;" % (key, value))
 
     ############################################################################
     # Resign opcode                                                            #
     ############################################################################
     if model.status in (WHITEWON, BLACKWON) and model.reason == WON_RESIGN:
-        file.write(" resign;")
+        handle.write(" resign;")
 
-    print("", file=file)
-    file.close()
+    print("", file=handle)
+    handle.close()
 
 
-def load(file):
-    return EpdFile([line.strip() for line in file if line])
+def load(handle):
+    return EpdFile(handle)
 
 
 class EpdFile(ChessFile):
-    def loadToModel(self, gameno, position, model=None):
+    def __init__(self, handle):
+        ChessFile.__init__(self, handle)
+
+        self.games = [self.create_rec(line.strip()) for line in handle if line]
+        self.count = len(self.games)
+
+    def create_rec(self, line):
+        rec = collections.defaultdict(str)
+        rec["Id"] = 0
+        rec["Offset"] = 0
+        rec["FEN"] = line
+
+        castling = rec["FEN"].split()[2]
+        for letter in castling:
+            if letter.upper() in "ABCDEFGH":
+                rec["Variant"] = FISCHERRANDOMCHESS
+                break
+
+        return rec
+
+    def loadToModel(self, rec, position, model=None):
         if not model:
             model = GameModel()
 
-        fieldlist = self.games[gameno].split(" ")
+        if "Variant" in rec:
+            model.variant = FischerandomBoard
+
+        fieldlist = rec["FEN"].split(" ")
         if len(fieldlist) == 4:
-            fen = self.games[gameno]
+            fen = rec["FEN"]
             opcodestr = ""
 
         elif len(fieldlist) > 4:
@@ -95,7 +116,7 @@ class EpdFile(ChessFile):
             raise LoadingError("EPD string can not have less than 4 field")
 
         opcodes = {}
-        for opcode in map(strip, opcodestr.split(";")):
+        for opcode in map(str.strip, opcodestr.split(";")):
             space = opcode.find(" ")
             if space == -1:
                 opcodes[opcode] = True
@@ -134,8 +155,8 @@ class EpdFile(ChessFile):
 
         return model
 
-    def get_player_names(self, gameno):
-        data = self.games[gameno]
+    def get_player_names(self, rec):
+        data = rec["FEN"]
 
         names = {}
 
