@@ -1,14 +1,11 @@
 import colorsys
-import re
 import sys
-import webbrowser
 import xml.etree.cElementTree as ET
 # from io import BytesIO
 
 from gi.repository import Gtk, Gdk, GObject, Pango
 from gi.repository.GdkPixbuf import Pixbuf
 
-from pychess.compat import PY3
 from pychess.System import conf
 from pychess.System.Log import log
 from pychess.System.prefix import addDataPrefix
@@ -141,7 +138,9 @@ METHODS = (
     (Gtk.ToggleButton, ("get_active", "set_active", "toggled")),
     (Gtk.CheckMenuItem, ("get_active", "set_active", "toggled")),
     (Gtk.Range, ("get_value", "set_value", "value-changed")),
-    (Gtk.TreeSortable, ("get_value", "set_value", "sort-column-changed")), )
+    (Gtk.TreeSortable, ("get_value", "set_value", "sort-column-changed")),
+    (Gtk.Paned, ("get_position", "set_position", "notify::position")),
+)
 
 
 def keep(widget, key, get_value_=None, set_value_=None, first_value=None):
@@ -170,10 +169,10 @@ def keep(widget, key, get_value_=None, set_value_=None, first_value=None):
 
     def setFromConf():
         try:
-            v = conf.getStrict(key)
+            v = conf.get(key)
         except TypeError:
             log.warning("uistuff.keep.setFromConf: Key '%s' from conf had the wrong type '%s', ignored" %
-                        (key, type(conf.getStrict(key))))
+                        (key, type(conf.get(key))))
             if first_value is not None:
                 conf.set(key, first_value)
             else:
@@ -182,7 +181,7 @@ def keep(widget, key, get_value_=None, set_value_=None, first_value=None):
             set_value(v)
 
     def callback(*args):
-        if not conf.hasKey(key) or conf.getStrict(key) != get_value():
+        if not conf.hasKey(key) or conf.get(key) != get_value():
             conf.set(key, get_value())
 
     widget.connect(signal, callback)
@@ -233,10 +232,10 @@ def loadDialogWidget(widget,
 
     if conf.hasKey(key):
         try:
-            v = conf.getStrict(key)
+            v = conf.get(key)
         except TypeError:
             log.warning("uistuff.loadDialogWidget: Key '%s' from conf had the wrong type '%s', ignored" %
-                        (key, type(conf.getStrict(key))))
+                        (key, type(conf.get(key))))
             if first_value is not None:
                 conf.set(key, first_value)
             else:
@@ -245,7 +244,7 @@ def loadDialogWidget(widget,
             set_value(v)
     elif first_value is not None:
         conf.set(key, first_value)
-        set_value(conf.getStrict(key))
+        set_value(conf.get(key))
     else:
         log.warning("Didn't load widget \"%s\": no conf value and no first_value arg" % widget_name)
 
@@ -271,7 +270,7 @@ def saveDialogWidget(widget, widget_name, config_number, get_value_=None):
     else:
         get_value = getattr(widget, getter)
 
-    if not conf.hasKey(key) or conf.getStrict(key) != get_value():
+    if not conf.hasKey(key) or conf.get(key) != get_value():
         conf.set(key, get_value())
 
 
@@ -313,8 +312,8 @@ def keepWindowSize(key,
         width, height = window.get_size_request()
 
         if conf.hasKey(key + "_width") and conf.hasKey(key + "_height"):
-            width = conf.getStrict(key + "_width")
-            height = conf.getStrict(key + "_height")
+            width = conf.get(key + "_width")
+            height = conf.get(key + "_height")
             log.debug("Resizing window to width=%s height=%s" %
                       (width, height))
             window.resize(width, height)
@@ -338,9 +337,10 @@ def keepWindowSize(key,
             window.resize(1, 1)
 
         if conf.hasKey(key + "_x") and conf.hasKey(key + "_y"):
-            log.debug("Moving window to x=%s y=%s" %
-                      (conf.getStrict(key + "_x"), conf.getStrict(key + "_y")))
-            window.move(conf.getStrict(key + "_x"), conf.getStrict(key + "_y"))
+            x = max(0, conf.get(key + "_x"))
+            y = max(0, conf.get(key + "_y"))
+            log.debug("Moving window to x=%s y=%s" % (x, y))
+            window.move(x, y)
 
         elif defaultPosition in (POSITION_CENTER, POSITION_GOLDEN):
             monitor_x, monitor_y, monitor_width, monitor_height = getMonitorBounds(
@@ -406,103 +406,6 @@ def makeYellow(box):
     onceWhenReady(box, cb)
 
 
-linkre = re.compile("http://(?:www\.)?\w+\.\w{2,4}[^\s]+")
-emailre = re.compile("[\w\.]+@[\w\.]+\.\w{2,4}")
-
-
-def initTexviewLinks(textview, text):
-    tags = []
-    textbuffer = textview.get_buffer()
-
-    while True:
-        linkmatch = linkre.search(text)
-        emailmatch = emailre.search(text)
-        if not linkmatch and not emailmatch:
-            textbuffer.insert(textbuffer.get_end_iter(), text)
-            break
-
-        if emailmatch and (not linkmatch or emailmatch.start() < linkmatch.start()):
-            e_start = emailmatch.start()
-            e_end = emailmatch.end()
-            msg_type = "email"
-        else:
-            e_start = linkmatch.start()
-            e_end = linkmatch.end()
-            if text[e_end - 1] == ".":
-                e_end -= 1
-            msg_type = "link"
-        textbuffer.insert(textbuffer.get_end_iter(), text[:e_start])
-
-        tag = textbuffer.create_tag(None,
-                                    foreground="blue",
-                                    underline=Pango.Underline.SINGLE)
-        tags.append([tag, text[e_start:e_end], msg_type, textbuffer.get_end_iter()])
-
-        textbuffer.insert_with_tags(textbuffer.get_end_iter(), text[e_start:e_end], tag)
-
-        tags[-1].append(textbuffer.get_end_iter())
-
-        text = text[e_end:]
-
-    def on_press_in_textview(textview, event):
-        tv_iter = textview.get_iter_at_location(int(event.x), int(event.y))
-        if not tv_iter:
-            return
-
-        # https://gramps-project.org/bugs/view.php?id=9335
-        if not isinstance(tv_iter, Gtk.TextIter):
-            tv_iter = tv_iter[1]
-
-        for tag, link, msg_type, s, e in tags:
-            if tv_iter.has_tag(tag):
-                tag.props.foreground = "red"
-                break
-
-    def on_release_in_textview(textview, event):
-        tv_iter = textview.get_iter_at_location(int(event.x), int(event.y))
-        if not tv_iter:
-            return
-
-        # https://gramps-project.org/bugs/view.php?id=9335
-        if not isinstance(tv_iter, Gtk.TextIter):
-            tv_iter = tv_iter[1]
-
-        for tag, link, msg_type, s, e in tags:
-            if tv_iter and tv_iter.has_tag(tag) and \
-                    tag.props.foreground_Gdk.red == 0xffff:
-                if msg_type == "link":
-                    webbrowser.open(link)
-                else:
-                    webbrowser.open("mailto:" + link)
-            tag.props.foreground = "blue"
-
-    stcursor = Gdk.Cursor(Gdk.CursorType.XTERM)
-    linkcursor = Gdk.Cursor(Gdk.CursorType.HAND2)
-
-    def on_motion_in_textview(textview, event):
-        # textview.get_window().get_pointer()
-        tv_iter = textview.get_iter_at_location(int(event.x), int(event.y))
-        if not tv_iter:
-            return
-
-        # https://gramps-project.org/bugs/view.php?id=9335
-        if not isinstance(tv_iter, Gtk.TextIter):
-            tv_iter = tv_iter[1]
-
-        for tag, link, msg_type, s, e in tags:
-            if tv_iter.has_tag(tag):
-                textview.get_window(Gtk.TextWindowType.TEXT).set_cursor(
-                    linkcursor)
-                break
-        else:
-            textview.get_window(Gtk.TextWindowType.TEXT).set_cursor(stcursor)
-
-    textview.connect("motion-notify-event", on_motion_in_textview)
-    textview.connect("leave_notify_event", on_motion_in_textview)
-    textview.connect("button_press_event", on_press_in_textview)
-    textview.connect("button_release_event", on_release_in_textview)
-
-
 no_gettext = False
 
 
@@ -512,7 +415,7 @@ class GladeWidgets:
 
     def __init__(self, filename):
         # TODO: remove this when upstream fixes translations with Python3+Windows
-        if PY3 and sys.platform == "win32" and not no_gettext:
+        if sys.platform == "win32" and not no_gettext:
             tree = ET.parse(addDataPrefix("glade/%s" % filename))
             for node in tree.iter():
                 if 'translatable' in node.attrib:
