@@ -1,16 +1,16 @@
-from __future__ import absolute_import
 from collections import defaultdict
-from threading import RLock, Thread
+import asyncio
 import traceback
 import datetime
+from queue import Queue
+from io import StringIO
 
 from gi.repository import GObject
 
-from pychess.compat import basestring, Queue, Empty, StringIO
 from pychess.Savers.ChessFile import LoadingError
 from pychess.Players.Player import PlayerIsDead, TurnInterrupt, InvalidMove
-from pychess.System import conf, fident
-from pychess.System.protoopen import protoopen, protosave, isWriteable
+from pychess.System import conf
+from pychess.System.protoopen import protoopen, protosave
 from pychess.System.Log import log
 from pychess.Utils.Move import Move
 from pychess.Utils.eco import get_eco
@@ -32,40 +32,7 @@ from .const import WAITING_TO_START, UNKNOWN_REASON, WHITE, ARTIFICIAL, RUNNING,
     DRAW_50MOVES
 
 
-def undolocked(f):
-    def newFunction(*args, **kw):
-        self = args[0]
-        log.debug("undolocked: adding func to queue: %s %s %s" % (
-            repr(f), repr(args), repr(kw)))
-        self.undoQueue.put((f, args, kw))
-
-        locked = self.undoLock.acquire(blocking=False)
-        if locked:
-            try:
-                while True:
-                    try:
-                        func, args, kw = self.undoQueue.get_nowait()
-                        log.debug("undolocked: running queued func: %s %s %s" % (
-                            repr(func), repr(args), repr(kw)))
-                        func(*args, **kw)
-                    except Empty:
-                        break
-            finally:
-                self.undoLock.release()
-
-    return newFunction
-
-
-def inthread(f):
-    def newFunction(*args, **kwargs):
-        thread = Thread(target=f, name=fident(f), args=args, kwargs=kwargs)
-        thread.daemon = True
-        thread.start()
-
-    return newFunction
-
-
-class GameModel(GObject.GObject, Thread):
+class GameModel(GObject.GObject):
     """ GameModel contains all available data on a chessgame.
         It also has the task of controlling players actions and moves """
 
@@ -120,12 +87,14 @@ class GameModel(GObject.GObject, Thread):
         "opening_changed": (GObject.SignalFlags.RUN_FIRST, None, ()),
         # variation_added is emitted if a variation was added.
         "variation_added": (GObject.SignalFlags.RUN_FIRST, None,
-                            (object, object, str, str)),
+                            (object, object)),
         # variation_extended is emitted if a new move was added to a variation.
         "variation_extended": (GObject.SignalFlags.RUN_FIRST, None,
                                (object, object)),
         # scores_changed is emitted if the analyzing scores was changed.
         "analysis_changed": (GObject.SignalFlags.RUN_FIRST, None, (int, )),
+        # analysis_finished is emitted if the game analyzing finished stepping on all moves.
+        "analysis_finished": (GObject.SignalFlags.RUN_FIRST, None, ()),
         # FICS games can get kibitz/whisper messages
         "message_received": (GObject.SignalFlags.RUN_FIRST, None, (str, str)),
         # FICS games can have observers
@@ -134,7 +103,6 @@ class GameModel(GObject.GObject, Thread):
 
     def __init__(self, timemodel=None, variant=NormalBoard):
         GObject.GObject.__init__(self)
-        Thread.__init__(self, name=fident(self.run))
         self.daemon = True
         self.variant = variant
         self.boards = [variant(setup=True)]
@@ -175,6 +143,8 @@ class GameModel(GObject.GObject, Thread):
         }
 
         self.endstatus = None
+        self.zero_reached_cid = None
+
         self.timed = self.timemodel.minutes != 0 or self.timemodel.gain != 0
         if self.timed:
             self.zero_reached_cid = self.timemodel.connect('zero_reached', self.zero_reached)
@@ -186,18 +156,20 @@ class GameModel(GObject.GObject, Thread):
             # loading from pgn. If you want to know the time left for a player,
             # check the time model.
 
-            # Keeps track of offers, so that accepts can be spotted
+        # Keeps track of offers, so that accepts can be spotted
         self.offers = {}
+
         # True if the game has been changed since last save
         self.needsSave = False
-        # The uri the current game was loaded from, or None if not a loaded
-        # game
+
+        # The uri the current game was loaded from, or None if not a loaded game
         self.uri = None
 
+        # Link to additiona info
+        self.info = None
+
         self.spectators = {}
 
-        self.applyingMoveLock = RLock()
-        self.undoLock = RLock()
         self.undoQueue = Queue()
 
     def zero_reached(self, timemodel, color):
@@ -259,9 +231,10 @@ class GameModel(GObject.GObject, Thread):
         else:
             return BLACK
 
+    @asyncio.coroutine
     def start_analyzer(self, analyzer_type):
         from pychess.Players.engineNest import init_engine
-        analyzer = init_engine(analyzer_type, self)
+        analyzer = yield from init_engine(analyzer_type, self)
         if analyzer is None:
             return
 
@@ -269,7 +242,6 @@ class GameModel(GObject.GObject, Thread):
         self.spectators[analyzer_type] = analyzer
         self.emit("analyzer_added", analyzer, analyzer_type)
         self.analyzer_cids[analyzer_type] = analyzer.connect("analyze", self.on_analyze)
-        return analyzer
 
     def remove_analyzer(self, analyzer_type):
         try:
@@ -286,9 +258,7 @@ class GameModel(GObject.GObject, Thread):
         try:
             analyzer = self.spectators[analyzer_type]
         except KeyError:
-            analyzer = self.start_analyzer(analyzer_type)
-            if analyzer is None:
-                return
+            return
 
         analyzer.resume()
         analyzer.setOptionInitialBoard(self)
@@ -303,11 +273,10 @@ class GameModel(GObject.GObject, Thread):
         analyzer.pause()
         self.emit("analyzer_paused", analyzer, analyzer_type)
 
+    @asyncio.coroutine
     def restart_analyzer(self, analyzer_type):
         self.remove_analyzer(analyzer_type)
-        self.start_analyzer(analyzer_type)
-        if self.isPlayingICSGame():
-            self.pause_analyzer(analyzer_type)
+        yield from self.start_analyzer(analyzer_type)
 
     def on_analyze(self, analyzer, analysis):
         if analysis and analysis[0] is not None:
@@ -428,8 +397,9 @@ class GameModel(GObject.GObject, Thread):
 
     def isPlayingICSGame(self):
         if self.players and self.status in (WAITING_TO_START, PAUSED, RUNNING):
-            if self.players[0].__type__ == LOCAL and self.players[1].__type__ == REMOTE or \
-               self.players[1].__type__ == LOCAL and self.players[0].__type__ == REMOTE:
+            if (self.players[0].__type__ == LOCAL and self.players[1].__type__ == REMOTE) or \
+               (self.players[1].__type__ == LOCAL and self.players[0].__type__ == REMOTE) or \
+               (self.players[1].__type__ == REMOTE and self.players[0].__type__ == REMOTE and self.examined):
                 return True
         return False
 
@@ -615,7 +585,7 @@ class GameModel(GObject.GObject, Thread):
             raise error
 
     def save(self, uri, saver, append, position=None):
-        if isinstance(uri, basestring):
+        if isinstance(uri, str):
             fileobj = protosave(uri, append)
             self.uri = uri
         else:
@@ -627,80 +597,78 @@ class GameModel(GObject.GObject, Thread):
 
     # Run stuff
 
-    def run(self):
-        log.debug("GameModel.run: Starting. self=%s" % self)
-        # Avoid racecondition when self.start is called while we are in
-        # self.end
-        if self.status != WAITING_TO_START:
-            return
+    def start(self):
 
-        if not self.isLocalGame():
-            self.timemodel.handle_gain = False
+        def coro():
+            log.debug("GameModel.run: Starting. self=%s" % self)
+            # Avoid racecondition when self.start is called while we are in
+            # self.end
+            if self.status != WAITING_TO_START:
+                return
 
-        self.status = RUNNING
+            if not self.isLocalGame():
+                self.timemodel.handle_gain = False
 
-        for player in self.players + list(self.spectators.values()):
-            player.start()
+            self.status = RUNNING
 
-        log.debug("GameModel.run: emitting 'game_started' self=%s" % self)
-        self.emit("game_started")
+            for player in self.players + list(self.spectators.values()):
+                event = asyncio.Event()
+                player.start(event)
+                yield from event.wait()
 
-        # Let GameModel end() itself on games started with loadAndStart()
-        self.checkStatus()
+            log.debug("GameModel.run: emitting 'game_started' self=%s" % self)
+            self.emit("game_started")
 
-        self.curColor = self.boards[-1].color
+            # Let GameModel end() itself on games started with loadAndStart()
+            self.checkStatus()
 
-        while self.status in (PAUSED, RUNNING, DRAW, WHITEWON, BLACKWON):
-            curPlayer = self.players[self.curColor]
+            self.curColor = self.boards[-1].color
 
-            if self.timed:
-                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: updating %s's time" % (
-                    id(self), str(self.players), str(self.ply), str(curPlayer)))
-                curPlayer.updateTime(
-                    self.timemodel.getPlayerTime(self.curColor),
-                    self.timemodel.getPlayerTime(1 - self.curColor))
+            while self.status in (PAUSED, RUNNING, DRAW, WHITEWON, BLACKWON):
+                curPlayer = self.players[self.curColor]
 
-            try:
-                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: calling %s.makeMove()" % (
-                    id(self), str(self.players), self.ply, str(curPlayer)))
-                if self.ply > self.lowply:
-                    move = curPlayer.makeMove(self.boards[-1], self.moves[-1],
-                                              self.boards[-2])
-                else:
-                    move = curPlayer.makeMove(self.boards[-1], None, None)
-                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: got move=%s from %s" % (
-                    id(self), str(self.players), self.ply, move, str(curPlayer)))
-            except PlayerIsDead as e:
-                if self.status in (WAITING_TO_START, PAUSED, RUNNING):
-                    stringio = StringIO()
-                    traceback.print_exc(file=stringio)
-                    error = stringio.getvalue()
-                    log.error(
-                        "GameModel.run: A Player died: player=%s error=%s\n%s"
-                        % (curPlayer, error, e))
-                    if self.curColor == WHITE:
-                        self.kill(WHITE_ENGINE_DIED)
+                if self.timed:
+                    log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: updating %s's time" % (
+                        id(self), str(self.players), str(self.ply), str(curPlayer)))
+                    curPlayer.updateTime(
+                        self.timemodel.getPlayerTime(self.curColor),
+                        self.timemodel.getPlayerTime(1 - self.curColor))
+                try:
+                    log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: calling %s.makeMove()" % (
+                        id(self), str(self.players), self.ply, str(curPlayer)))
+                    if self.ply > self.lowply:
+                        move = yield from curPlayer.makeMove(self.boards[-1], self.moves[-1], self.boards[-2])
                     else:
-                        self.kill(BLACK_ENGINE_DIED)
-                break
-            except InvalidMove as e:
-                if self.curColor == WHITE:
-                    self.end(BLACKWON, WON_ADJUDICATION)
-                else:
-                    self.end(WHITEWON, WON_ADJUDICATION)
-                break
-            except TurnInterrupt:
-                log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: TurnInterrupt" % (
-                    id(self), str(self.players), self.ply))
-                self.curColor = self.boards[-1].color
-                continue
+                        move = yield from curPlayer.makeMove(self.boards[-1], None, None)
+                    log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: got move=%s from %s" % (
+                        id(self), str(self.players), self.ply, move, str(curPlayer)))
+                except PlayerIsDead as e:
+                    if self.status in (WAITING_TO_START, PAUSED, RUNNING):
+                        stringio = StringIO()
+                        traceback.print_exc(file=stringio)
+                        error = stringio.getvalue()
+                        log.error(
+                            "GameModel.run: A Player died: player=%s error=%s\n%s"
+                            % (curPlayer, error, e))
+                        if self.curColor == WHITE:
+                            self.kill(WHITE_ENGINE_DIED)
+                        else:
+                            self.kill(BLACK_ENGINE_DIED)
+                    break
+                except InvalidMove as e:
+                    if self.curColor == WHITE:
+                        self.end(BLACKWON, WON_ADJUDICATION)
+                    else:
+                        self.end(WHITEWON, WON_ADJUDICATION)
+                    break
+                except TurnInterrupt:
+                    log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: TurnInterrupt" % (
+                        id(self), str(self.players), self.ply))
+                    self.curColor = self.boards[-1].color
+                    continue
 
-            log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: acquiring self.applyingMoveLock" % (
-                id(self), str(self.players), self.ply))
-            assert isinstance(move, Move), "%s" % repr(move)
+                assert isinstance(move, Move), "%s" % repr(move)
 
-            self.applyingMoveLock.acquire()
-            try:
                 log.debug("GameModel.run: id=%s, players=%s, self.ply=%s: applying move=%s" % (
                     id(self), str(self.players), self.ply, str(move)))
                 self.needsSave = True
@@ -733,9 +701,7 @@ class GameModel(GObject.GObject, Thread):
                 self.checkStatus()
                 self.curColor = 1 - self.curColor
 
-            finally:
-                log.debug("GameModel.run: releasing self.applyingMoveLock")
-                self.applyingMoveLock.release()
+        asyncio.async(coro())
 
     def checkStatus(self):
         """ Updates self.status so it fits with what getStatus(boards[-1])
@@ -785,17 +751,12 @@ class GameModel(GObject.GObject, Thread):
             if self.timed:
                 self.timemodel.pause()
 
-    @inthread
     def pause(self):
         """ Players will raise NotImplementedError if they doesn't support
             pause. Spectators will be ignored. """
 
-        self.applyingMoveLock.acquire()
-        try:
-            self.__pause()
-            self.status = PAUSED
-        finally:
-            self.applyingMoveLock.release()
+        self.__pause()
+        self.status = PAUSED
         self.emit("game_paused")
 
     def __resume(self):
@@ -805,14 +766,9 @@ class GameModel(GObject.GObject, Thread):
             self.timemodel.resume()
         self.emit("game_resumed")
 
-    @inthread
     def resume(self):
-        self.applyingMoveLock.acquire()
-        try:
-            self.status = RUNNING
-            self.__resume()
-        finally:
-            self.applyingMoveLock.release()
+        self.status = RUNNING
+        self.__resume()
 
     def end(self, status, reason):
         if self.status not in UNFINISHED_STATES:
@@ -868,7 +824,8 @@ class GameModel(GObject.GObject, Thread):
                 self.timemodel.end()
                 log.debug("GameModel.terminate: <- timemodel.end() %s" %
                           repr(self.timemodel))
-                self.timemodel.disconnect(self.zero_reached_cid)
+                if self.zero_reached_cid is not None:
+                    self.timemodel.disconnect(self.zero_reached_cid)
 
         # ICGameModel may did this if game was a FICS game
         if self.connections is not None:
@@ -883,8 +840,6 @@ class GameModel(GObject.GObject, Thread):
 
     # Other stuff
 
-    @inthread
-    @undolocked
     def undoMoves(self, moves):
         """ Undo and remove moves number of moves from the game history from
             the GameModel, players, and any spectators """
@@ -900,32 +855,25 @@ class GameModel(GObject.GObject, Thread):
 
         log.debug("GameModel.undoMoves: players=%s, self.ply=%s, moves=%s, board=%s" % (
                   repr(self.players), self.ply, moves, self.boards[-1]))
-        log.debug("GameModel.undoMoves: acquiring self.applyingMoveLock")
-        self.applyingMoveLock.acquire()
-        log.debug("GameModel.undoMoves: self.applyingMoveLock acquired")
-        try:
-            self.emit("moves_undoing", moves)
-            self.needsSave = True
+        self.emit("moves_undoing", moves)
+        self.needsSave = True
 
-            self.boards = self.variations[0]
-            del self.boards[-moves:]
-            del self.moves[-moves:]
-            self.boards[-1].board.next = None
+        self.boards = self.variations[0]
+        del self.boards[-moves:]
+        del self.moves[-moves:]
+        self.boards[-1].board.next = None
 
-            for player in self.players:
-                player.playerUndoMoves(moves, self)
-            for spectator in self.spectators.values():
-                spectator.spectatorUndoMoves(moves, self)
+        for player in self.players:
+            player.playerUndoMoves(moves, self)
+        for spectator in self.spectators.values():
+            spectator.spectatorUndoMoves(moves, self)
 
-            log.debug("GameModel.undoMoves: undoing timemodel")
-            if self.timed:
-                self.timemodel.undoMoves(moves)
+        log.debug("GameModel.undoMoves: undoing timemodel")
+        if self.timed:
+            self.timemodel.undoMoves(moves)
 
-            self.checkStatus()
-            self.setOpening()
-        finally:
-            log.debug("GameModel.undoMoves: releasing self.applyingMoveLock")
-            self.applyingMoveLock.release()
+        self.checkStatus()
+        self.setOpening()
 
         self.emit("moves_undone", moves)
 
@@ -934,15 +882,22 @@ class GameModel(GObject.GObject, Thread):
             return False
         if self.needsSave:
             return True
-        if not self.uri or not isWriteable(self.uri):
-            return True
+        # what was this for?
+        # if not self.uri or not isWriteable(self.uri):
+            # return True
         return False
 
-    def add_variation(self, board, moves, comment="", score=""):
+    def add_variation(self, board, moves, comment="", score="", emit=True):
         board0 = board
         board = board0.clone()
         board.board.prev = None
 
+        # this prevents annotation panel node searches to find this instead of board0
+        board.board.hash = -1
+
+        if comment:
+            board.board.children.append(comment)
+
         variation = [board]
 
         for move in moves:
@@ -967,6 +922,8 @@ class GameModel(GObject.GObject, Thread):
 
         board0.board.next.children.append(
             [vboard.board for vboard in variation])
+        if score:
+            variation[-1].board.children.append(score)
 
         head = None
         for vari in self.variations:
@@ -977,8 +934,8 @@ class GameModel(GObject.GObject, Thread):
         variation[0] = board0
         self.variations.append(head[:board0.ply - self.lowply] + variation)
         self.needsSave = True
-        self.emit("variation_added", board0.board.next.children[-1],
-                  board0.board.next, comment, score)
+        if emit:
+            self.emit("variation_added", board0.board.next.children[-1], board0.board.next)
         return self.variations[-1]
 
     def add_move2variation(self, board, move, variationIdx):
