@@ -1,7 +1,6 @@
 """ This module contains chess logic functins for the pychess client. They are
     based upon the lutils modules, but supports standard object types and is
     therefore not as fast. """
-from __future__ import absolute_import
 
 from .lutils import lmovegen
 from .lutils.validator import validateMove
@@ -13,7 +12,7 @@ from .const import LOSERSCHESS, WHITE, WHITEWON, BLACKWON, WON_NOMATERIAL, KING,
     SUICIDECHESS, ATOMICCHESS, WON_KINGEXPLODE, KINGOFTHEHILLCHESS, BLACK, DRAW, \
     WON_KINGINCENTER, THREECHECKCHESS, WON_THREECHECK, WON_MATE, DRAW_STALEMATE, \
     DRAW_INSUFFICIENT, DRAW_EQUALMATERIAL, WON_LESSMATERIAL, DRAW_REPITITION, \
-    DRAW_50MOVES, RUNNING, ENPASSANT, UNKNOWN_REASON
+    WON_KINGINEIGHTROW, RACINGKINGSCHESS, DRAW_50MOVES, RUNNING, ENPASSANT, UNKNOWN_REASON
 
 from .lutils.bitboard import iterBits
 from .lutils.attack import getAttacks
@@ -22,6 +21,7 @@ from pychess.Variants.losers import testKingOnly
 from pychess.Variants.atomic import kingExplode
 from pychess.Variants.kingofthehill import testKingInCenter
 from pychess.Variants.threecheck import checkCount
+from pychess.Variants.racingkings import testKingInEightRow
 
 
 def getDestinationCords(board, cord):
@@ -85,6 +85,13 @@ def getStatus(board):
             else:
                 status = BLACKWON
             return status, WON_THREECHECK
+    elif board.variant == RACINGKINGSCHESS:
+        if testKingInEightRow(lboard):
+            if board.color == BLACK:
+                status = WHITEWON
+            else:
+                status = BLACKWON
+            return status, WON_KINGINEIGHTROW
     else:
         if ldraw.testMaterial(lboard):
             return DRAW, DRAW_INSUFFICIENT
@@ -199,6 +206,12 @@ def validate(board, move):
             return True
         else:
             return standard_validate(board, move)
+    elif board.variant == RACINGKINGSCHESS:
+        # Giving check is forbidden
+        if board.board.willGiveCheck(move.move):
+            return False
+        else:
+            return standard_validate(board, move)
     else:
         return standard_validate(board, move)
 
