@@ -1,12 +1,9 @@
 # -*- coding: UTF-8 -*-
-from __future__ import absolute_import
-from __future__ import unicode_literals
 
 from .ldata import bitPosArray, fileBits, rankBits
 from .bitboard import firstBit
 from .validator import validateMove
 
-from pychess.compat import unichr, unicode
 from pychess.Utils.const import SAN, AN, LAN, ENPASSANT, EMPTY, PAWN, KING_CASTLE, QUEEN_CASTLE,\
     reprFile, reprRank, chr2Sign, cordDic, reprSign, reprCord, reprSignSittuyin, reprSignMakruk,\
     QUEEN, KNIGHT, BISHOP, ROOK, KING, NORMALCHESS, NORMAL_MOVE, PROMOTIONS, WHITE, BLACK, DROP,\
@@ -338,9 +335,7 @@ def parseSAN(board, san):
         notat = notat[1:]
     else:
         piece = PAWN
-        if notat[
-                -
-                1] in "18" and flag == NORMAL_MOVE and board.variant != SITTUYINCHESS:
+        if notat[-1] in "18" and flag == NORMAL_MOVE and board.variant != SITTUYINCHESS:
             raise ParsingError(
                 san, _("promotion move without promoted piece is incorrect"),
                 board.asFen())
@@ -356,7 +351,12 @@ def parseSAN(board, san):
         if piece == PAWN:
             # If a pawn is attacking an empty cord, we assue it an enpassant
             if board.arBoard[tcord] == EMPTY:
-                flag = ENPASSANT
+                if (color == BLACK and 2 * 8 <= tcord < 3 * 8) or (color == WHITE and 5 * 8 <= tcord < 6 * 8):
+                    flag = ENPASSANT
+                else:
+                    raise ParsingError(
+                        san, _("pawn capture without target piece is invalid"),
+                        board.asFen())
     else:
         if not notat[-2:] in cordDic:
             raise ParsingError(san, _("the end cord (%s) is incorrect") %
@@ -581,7 +581,15 @@ def parseAN(board, an):
         # The a7a8=q variant
         flag = chr2Sign[an[5].lower()] + 2
     elif board.arBoard[fcord] == KING:
-        if board.variant == FISCHERRANDOMCHESS and board.arBoard[
+        if fcord - tcord == 2:
+            flag = QUEEN_CASTLE
+            if board.variant == FISCHERRANDOMCHESS:
+                tcord = board.ini_rooks[board.color][0]
+        elif fcord - tcord == -2:
+            flag = KING_CASTLE
+            if board.variant == FISCHERRANDOMCHESS:
+                tcord = board.ini_rooks[board.color][1]
+        elif board.arBoard[
                 tcord] == ROOK:
             color = board.color
             friends = board.friends[color]
@@ -590,11 +598,6 @@ def parseAN(board, an):
                     flag = QUEEN_CASTLE
                 else:
                     flag = KING_CASTLE
-
-        elif fcord - tcord == 2:
-            flag = QUEEN_CASTLE
-        elif fcord - tcord == -2:
-            flag = KING_CASTLE
         else:
             flag = NORMAL_MOVE
     elif board.arBoard[fcord] == PAWN and board.arBoard[tcord] == EMPTY and \
@@ -642,7 +645,7 @@ san2BlackFanDic = {
 def toFAN(board, move):
     """ Returns a Figurine Algebraic Notation string of a move """
 
-    san = unicode(toSAN(board, move))
+    san = toSAN(board, move)
     if board.color == WHITE:
         return san.translate(san2WhiteFanDic)
     else:
@@ -654,9 +657,9 @@ def toFAN(board, move):
 
 fan2SanDic = {}
 for k, v in san2WhiteFanDic.items():
-    fan2SanDic[ord(v)] = unichr(k)
+    fan2SanDic[ord(v)] = chr(k)
 for k, v in san2BlackFanDic.items():
-    fan2SanDic[ord(v)] = unichr(k)
+    fan2SanDic[ord(v)] = chr(k)
 
 
 def parseFAN(board, fan):
