@@ -3,9 +3,7 @@
 import datetime
 from gi.repository import GLib, GObject
 
-from pychess.compat import unicode
 from pychess.System.Log import log
-from pychess.System.idle_add import idle_add
 from pychess.Utils.IconLoader import load_icon
 from pychess.Utils.const import ADJOURNED, WHITE, BLACK, UNSUPPORTED
 
@@ -24,7 +22,6 @@ class AlreadyExistException(Exception):
     pass
 
 
-@idle_add
 def make_sensitive_if_available(button, player):
     if player.isAvailableForGame():
         button.set_property("sensitive", True)
@@ -35,7 +32,6 @@ def make_sensitive_if_available(button, player):
                             {"player": player.name, "status": player.display_status.lower()})
 
 
-@idle_add
 def make_sensitive_if_playing(button, player):
     status = player.display_status.lower()
     if player.status == IC_STATUS_PLAYING:
@@ -351,7 +347,11 @@ class FICSPlayer(GObject.GObject):
         ratingtotal = 0
         numratings = 0
         for ratingtype in RATING_TYPES:
-            if self.ratings[ratingtype] == 0:
+            try:
+                if self.ratings[ratingtype] == 0:
+                    continue
+            except IndexError:
+                print(ratingtype, RATING_TYPES, self.ratings)
                 continue
             if self.deviations[ratingtype] == DEVIATION_NONE:
                 ratingtotal += self.ratings[ratingtype] * 3
@@ -707,11 +707,11 @@ def get_rating_range_display_text(rmin=0, rmax=9999):
     if rmin > 0:
         text = "%d" % rmin
         if rmax == 9999:
-            text += unicode("↑")
+            text += u"↑"
         else:
             text += "-%d" % rmax
     elif rmax != 9999:
-        text = unicode("%d↓" % rmax)
+        text = u"%d↓" % rmax
     else:
         text = None
     return text
