@@ -1,8 +1,6 @@
-from __future__ import print_function
+import asyncio
+from io import StringIO
 
-import threading
-
-from pychess.compat import StringIO
 from pychess.System.Log import log
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.Offer import Offer
@@ -22,11 +20,12 @@ class ICGameModel(GameModel):
         self.connection = connection
         self.ficsgame = ficsgame
         self.ficsplayers = (ficsgame.wplayer, ficsgame.bplayer)
-        self.gmwidg_ready = threading.Event()
+        self.gmwidg_ready = asyncio.Event()
+        self.kibitz_task = None
 
         connections = self.connections
         connections[connection.bm].append(connection.bm.connect(
-            "boardUpdate", self.onBoardUpdate))
+            "exGameBackward", self.onExGameBackward))
         connections[connection.bm].append(connection.bm.connect(
             "timesUpdate", self.onTimesUpdate))
         connections[connection.bm].append(connection.bm.connect(
@@ -52,9 +51,9 @@ class ICGameModel(GameModel):
 
         rated = "rated" if ficsgame.rated else "unrated"
         # This is in the format that ficsgames.org writes these PGN headers
-        self.tags["Event"] = "FICS %s %s game" % (rated,
-                                                  ficsgame.game_type.fics_name)
-        self.tags["Site"] = "freechess.org"
+        ics = "ICC" if self.connection.ICC else "FICS"
+        self.tags["Event"] = "%s %s %s game" % (ics, rated, ficsgame.game_type.fics_name)
+        self.tags["Site"] = "chessclub.com" if self.connection.ICC else "freechess.org"
 
     def __repr__(self):
         string = GameModel.__repr__(self)
@@ -117,7 +116,11 @@ class ICGameModel(GameModel):
                 return False
         return True
 
-    def onBoardUpdate(self, bm, gameno, ply, curcol, lastmove, fen, wname,
+    def onExGameBackward(self, bm, gameno, ply):
+        if gameno == self.ficsgame.gameno:
+            self.undoMoves(ply)
+
+    def onBoardUpdate(self, gameno, ply, curcol, lastmove, fen, wname,
                       bname, wms, bms):
         log.debug(("ICGameModel.onBoardUpdate: id=%s self.ply=%s self.players=%s gameno=%s " +
                   "wname=%s bname=%s ply=%s curcol=%s lastmove=%s fen=%s wms=%s bms=%s") %
@@ -142,16 +145,21 @@ class ICGameModel(GameModel):
 
             self.timemodel.updatePlayer(WHITE, wms / 1000.)
             self.timemodel.updatePlayer(BLACK, bms / 1000.)
-
         if lastmove is None:
             if bname != self.tags["Black"]:
                 self.tags["Black"] = self.players[
                     BLACK].name = self.ficsplayers[BLACK].name = bname
                 self.emit("players_changed")
+                curPlayer = self.players[self.curColor]
+                curPlayer.resetPosition()
+
             if wname != self.tags["White"]:
                 self.tags["White"] = self.players[
                     WHITE].name = self.ficsplayers[WHITE].name = wname
                 self.emit("players_changed")
+                curPlayer = self.players[self.curColor]
+                curPlayer.resetPosition()
+
             if self.boards[-1].asFen() != fen:
                 self.status = RUNNING
                 self.loadAndStart(
@@ -254,11 +262,13 @@ class ICGameModel(GameModel):
     ############################################################################
 
     def onKibitzMessage(self, cm, name, gameno, text):
-        if not self.gmwidg_ready.is_set():
-            self.gmwidg_ready.wait()
-        if gameno != self.ficsgame.gameno:
-            return
-        self.emit("message_received", name, text)
+        def coro():
+            if not self.gmwidg_ready.is_set():
+                yield from self.gmwidg_ready.wait()
+            if gameno != self.ficsgame.gameno:
+                return
+            self.emit("message_received", name, text)
+        self.kibitz_task = asyncio.async(coro())
 
     def onWhisperMessage(self, cm, name, gameno, text):
         if gameno != self.ficsgame.gameno:
@@ -353,6 +363,8 @@ class ICGameModel(GameModel):
 
         self.connections = None
         GameModel.terminate(self)
+        if self.kibitz_task is not None:
+            self.kibitz_task.cancel()
 
     def goFirst(self):
         self.connection.client.run_command("backward 999")
