@@ -1,5 +1,5 @@
-from __future__ import absolute_import
 
+import asyncio
 import re
 import socket
 import webbrowser
@@ -10,7 +10,7 @@ from gi.repository import Gtk
 from gi.repository import GObject
 
 from pychess.System import uistuff, conf
-from pychess.System.idle_add import idle_add
+from pychess.widgets import mainwindow
 from .FICSConnection import FICSMainConnection, FICSHelperConnection, LogOnException
 from .ICLounge import ICLounge
 
@@ -22,11 +22,9 @@ dialog = None
 
 def run():
     global dialog
+    dialog.widgets["fics_logon"].set_transient_for(mainwindow())
 
-    if not dialog:
-        dialog = ICLogon()
-        dialog.show()
-    elif dialog.lounge:
+    if dialog.lounge:
         dialog.lounge.present()
     else:
         dialog.present()
@@ -43,24 +41,12 @@ class AutoLogoutException(Exception):
     pass
 
 
-def get_user_names(value=None):
-    """ Split and return usernameEntry config item into registered and guest username
-    """
-    if value is not None:
-        names = value.split("|")
-    else:
-        names = conf.get("usernameEntry", "").split("|")
-    if len(names) == 0:
-        names = ["", ""]
-    elif len(names) < 2:
-        names.append(names[0])
-    return names
-
-
 class ICLogon(object):
     def __init__(self):
         self.connection = None
         self.helperconn = None
+        self.connection_task = None
+        self.helperconn_task = None
         self.lounge = None
         self.canceled = False
         self.cids = defaultdict(list)
@@ -72,49 +58,23 @@ class ICLogon(object):
             'key-press-event',
             lambda w, e: e.keyval == Gdk.KEY_Escape and w.hide())
 
-        def on_logOnAsGuest_toggled(check):
-            names = get_user_names()
-            self.widgets["nameEntry"].set_text(names[1] if check.get_active()
-                                               else names[0])
-            self.widgets["passwordLabel"].set_sensitive(not check.get_active())
-            self.widgets["passEntry"].set_sensitive(not check.get_active())
+        self.ics = "FICS"
+        self.as_guest = self.widgets["logOnAsGuest"]
 
-        self.widgets["logOnAsGuest"].connect("toggled",
-                                             on_logOnAsGuest_toggled)
-        uistuff.keep(self.widgets["logOnAsGuest"], "asGuestCheck")
+        self.widgets["logOnAsGuest"].connect("toggled", self.on_logOnAsGuest_toggled)
 
-        as_guest = self.widgets["logOnAsGuest"]
+        def on_username_changed(widget):
+            conf.set("usernameEntry", self.user_name_get_value(widget), section=self.ics)
+        self.widgets["nameEntry"].connect("changed", on_username_changed)
 
-        def user_name_get_value(entry):
-            names = get_user_names()
-            if as_guest.get_active():
-                text = "%s|%s" % (names[0], entry.get_text())
-            else:
-                text = "%s|%s" % (entry.get_text(), names[1])
-            return text
+        def on_password_changed(widget):
+            conf.set("passwordEntry", widget.get_text(), section=self.ics)
+        self.widgets["passEntry"].connect("changed", on_password_changed)
 
-        def user_name_set_value(entry, value):
-            names = get_user_names(value=value)
-            if as_guest.get_active():
-                entry.set_text(names[1])
-            else:
-                entry.set_text(names[0])
+        def on_host_changed(widget):
+            conf.set("hostEntry", self.host_get_value(widget), section=self.ics)
+        self.widgets["hostEntry"].connect("changed", on_host_changed)
 
-        uistuff.keep(self.widgets["nameEntry"], "usernameEntry",
-                     user_name_get_value, user_name_set_value)
-        uistuff.keep(self.widgets["passEntry"], "passwordEntry")
-
-        # workaround to Can't type IP to FICS login dialog
-        # https://github.com/pychess/pychess/issues/1360
-        def host_get_value(entry):
-            return entry.get_text().replace(".", "|")
-
-        def host_set_value(entry, value):
-            entry.set_text(str(value).replace("|", "."))
-
-        uistuff.keep(self.widgets["hostEntry"], "hostEntry", host_get_value, host_set_value)
-
-        uistuff.keep(self.widgets["autoLogin"], "autoLogin")
         self.infobar = Gtk.InfoBar()
         self.infobar.set_message_type(Gtk.MessageType.WARNING)
         # self.widgets["messagePanelHBox"].pack_start(self.infobar,
@@ -128,6 +88,66 @@ class ICLogon(object):
 
         self.widgets["progressbar"].set_show_text(True)
 
+    def get_user_names(self, value=None):
+        """ Split and return usernameEntry config item into registered and guest username
+        """
+        if value is not None:
+            names = value.split("|")
+        else:
+            names = conf.get("usernameEntry", "", section=self.ics).split("|")
+        if len(names) == 0:
+            names = ["", ""]
+        elif len(names) < 2:
+            names.append(names[0])
+        return names
+
+    def user_name_get_value(self, entry):
+        names = self.get_user_names()
+        if self.as_guest.get_active():
+            text = "%s|%s" % (names[0], entry.get_text())
+        else:
+            text = "%s|%s" % (entry.get_text(), names[1])
+        return text
+
+    def user_name_set_value(self, entry, value):
+        names = self.get_user_names(value=value)
+        if self.as_guest.get_active():
+            entry.set_text(names[1])
+        else:
+            entry.set_text(names[0])
+
+    # workaround to FICS Password input doesnt handle strings starting with a number
+    # https://github.com/pychess/pychess/issues/1375
+    def password_set_value(self, entry, value):
+        entry.set_text(str(value))
+
+    # workaround to Can't type IP to FICS login dialog
+    # https://github.com/pychess/pychess/issues/1360
+    def host_get_value(self, entry):
+        return entry.get_text().replace(".", "|")
+
+    def host_set_value(self, entry, value):
+        entry.set_text(str(value).replace("|", "."))
+
+    def on_logOnAsGuest_toggled(self, widget):
+        names = self.get_user_names()
+        self.widgets["nameEntry"].set_text(names[1] if widget.get_active() else names[0])
+        self.widgets["passwordLabel"].set_sensitive(not widget.get_active())
+        self.widgets["passEntry"].set_sensitive(not widget.get_active())
+        conf.set("asGuestCheck", widget.get_active(), section=self.ics)
+
+    def on_ics_combo_changed(self, combo):
+        tree_iter = combo.get_active_iter()
+        if tree_iter is not None:
+            model = combo.get_model()
+            self.ics = model[tree_iter][0]
+            # print("Selected: %s" % self.ics)
+            self.widgets["logOnAsGuest"].set_active(conf.get("asGuestCheck", False, section=self.ics))
+            self.user_name_set_value(self.widgets["nameEntry"], conf.get("usernameEntry", "", section=self.ics))
+            self.password_set_value(self.widgets["passEntry"], conf.get("passwordEntry", "", section=self.ics))
+            default_host = "freechess.org" if self.ics == "FICS" else "chessclub.com"
+            self.host_set_value(self.widgets["hostEntry"], conf.get("hostEntry", default_host, section=self.ics))
+
     def _disconnect(self):
         for obj in self.cids:
             for cid in self.cids[obj]:
@@ -137,9 +157,13 @@ class ICLogon(object):
 
         if self.connection is not None:
             self.connection.close()
+            if not self.connection_task.cancelled():
+                self.connection_task.cancel()
             self.connection = None
         if self.helperconn is not None:
             self.helperconn.close()
+            if not self.helperconn_task.cancelled():
+                self.helperconn_task.cancel()
             self.helperconn = None
         self.lounge = None
 
@@ -234,7 +258,10 @@ class ICLogon(object):
         self.widgets["messagePanel"].show_all()
 
     def onCreateNew(self, button):
-        webbrowser.open("http://www.freechess.org/Register/index.html")
+        if self.widgets["hostEntry"].get_text() == "chessclub.com":
+            webbrowser.open("https://store.chessclub.com/customer/account/create/")
+        else:
+            webbrowser.open("http://www.freechess.org/Register/index.html")
 
     def onConnectClicked(self, button):
         self.canceled = False
@@ -267,16 +294,18 @@ class ICLogon(object):
                                  ("connectingMsg", self.showMessage)):
             self.cids[self.connection].append(self.connection.connect(
                 signal, callback))
-        self.connection.start()
+        self.main_connected_event = asyncio.Event()
+        self.connection_task = asyncio.async(self.connection.start())
 
-        self.helperconn = FICSHelperConnection(self.connection, self.host, ports)
-        self.helperconn.connect("error", self.onHelperConnectionError)
-        self.helperconn.start()
+        # guest users are rather limited on ICC (helper connection is useless)
+        if self.host not in ("localhost", "chessclub.com"):
+            self.helperconn = FICSHelperConnection(self.connection, self.host, ports)
+            self.helperconn.connect("error", self.onHelperConnectionError)
+            self.helperconn_task = asyncio.async(self.helperconn.start())
 
-    @idle_add
     def onHelperConnectionError(self, connection, error):
         if self.helperconn is not None:
-            dialog = Gtk.MessageDialog(type=Gtk.MessageType.QUESTION,
+            dialog = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.QUESTION,
                                        buttons=Gtk.ButtonsType.YES_NO)
             dialog.set_markup(_("Guest logins disabled by FICS server"))
             text = "PyChess can maintain users status and games list only if it changes\n\
@@ -291,10 +320,18 @@ class ICLogon(object):
             self.helperconn = None
 
             set_user_vars = response == Gtk.ResponseType.YES
-            self.connection.start_helper_manager(set_user_vars)
 
-    @idle_add
+            @asyncio.coroutine
+            def coro():
+                yield from self.main_connected_event.wait()
+                self.connection.start_helper_manager(set_user_vars)
+            asyncio.async(coro())
+
     def onConnected(self, connection):
+        self.main_connected_event.set()
+        if connection.ICC:
+            self.connection.start_helper_manager(True)
+
         self.lounge = ICLounge(connection, self.helperconn, self.host)
         self.hide()
         self.lounge.show()
@@ -306,7 +343,6 @@ class ICLogon(object):
         self.showNormal()
         self.widgets["messagePanel"].hide()
 
-    @idle_add
     def onCancel(self, widget, hide):
         self.canceled = True
 
@@ -316,18 +352,15 @@ class ICLogon(object):
         if hide:
             self.widgets["fics_logon"].hide()
 
-    @idle_add
     def onConnectionError(self, connection, error):
         self._disconnect()
         if not self.canceled:
             self.showError(connection, error)
             self.present()
 
-    @idle_add
     def onLogout(self, connection):
         self._disconnect()
 
-    @idle_add
     def onAutologout(self, connection):
         self._disconnect()
         self.showError(connection, AutoLogoutException())
