@@ -14,17 +14,28 @@ IC_POS_ISOLATED, IC_POS_OBSERVING_EXAMINATION, IC_POS_EXAMINATING, \
 # RatingType
 TYPE_BLITZ, TYPE_STANDARD, TYPE_LIGHTNING, TYPE_WILD, \
     TYPE_BUGHOUSE, TYPE_CRAZYHOUSE, TYPE_SUICIDE, TYPE_LOSERS, TYPE_ATOMIC, \
-    TYPE_UNTIMED, TYPE_EXAMINED, TYPE_OTHER = range(12)
+    TYPE_BULLET, TYPE_ONE_MINUTE, TYPE_THREE_MINUTE, TYPE_FIVE_MINUTE, \
+    TYPE_FIFTEEN_MINUTE, TYPE_FORTYFIVE_MINUTE, TYPE_CHESS960, \
+    TYPE_UNTIMED, TYPE_EXAMINED, TYPE_OTHER = range(19)
 
 RATING_TYPES = (TYPE_BLITZ,
                 TYPE_STANDARD,
                 TYPE_LIGHTNING,
+                TYPE_BULLET,
+                TYPE_ONE_MINUTE,
+                TYPE_THREE_MINUTE,
+                TYPE_FIVE_MINUTE,
+                TYPE_FIFTEEN_MINUTE,
+                TYPE_FORTYFIVE_MINUTE,
                 TYPE_ATOMIC,
                 TYPE_BUGHOUSE,
                 TYPE_CRAZYHOUSE,
                 TYPE_LOSERS,
                 TYPE_SUICIDE,
-                TYPE_WILD, )
+                TYPE_WILD,
+                TYPE_CHESS960,
+                TYPE_UNTIMED,
+                )
 
 # Rating deviations
 DEVIATION_NONE, DEVIATION_ESTIMATED, DEVIATION_PROVISIONAL = range(3)
@@ -122,11 +133,17 @@ class WildGameType(VariantGameType):
     def instances(cls):
         return cls._instances
 
-
+# FICS game types
 GAME_TYPES = {
     "blitz": NormalGameType("blitz", "b", TYPE_BLITZ, _("Blitz")),
     "standard": NormalGameType("standard", "s", TYPE_STANDARD, _("Standard")),
     "lightning": NormalGameType("lightning", "l", TYPE_LIGHTNING, _("Lightning")),
+    "1-minute": NormalGameType("1-minute", "o", TYPE_ONE_MINUTE, _("1-minute")),
+    "3-minute": NormalGameType("3-minute", "M", TYPE_THREE_MINUTE, _("3-minute")),
+    "5-minute": NormalGameType("5-minute", "f", TYPE_FIVE_MINUTE, _("5-minute")),
+    "15-minute": NormalGameType("15-minute", "F", TYPE_FIFTEEN_MINUTE, _("15-minute")),
+    "45-minute": NormalGameType("45-minute", "J", TYPE_FORTYFIVE_MINUTE, _("45-minute")),
+    "chess960": NormalGameType("chess960", "K", TYPE_CHESS960, _("Chess960")),
     "untimed": NormalGameType("untimed", "u", TYPE_UNTIMED, _("Untimed")),
     "examined": NormalGameType("examined", "e", TYPE_EXAMINED, _("Examined")),
     "nonstandard": NormalGameType("nonstandard", "n", TYPE_OTHER, _("Other")),
@@ -176,6 +193,13 @@ for key in GAME_TYPES_BY_SHORT_FICS_NAME:
         GAME_TYPES_BY_SHORT_FICS_NAME[key]
 
 
+# Finally add conflicting ICC game type
+GAME_TYPES["bullet"] = NormalGameType("bullet", "B", TYPE_BULLET, _("Bullet"))
+GAME_TYPES_BY_FICS_NAME["bullet"] = GAME_TYPES["bullet"]
+GAME_TYPES_BY_RATING_TYPE[TYPE_BULLET] = GAME_TYPES["bullet"]
+# GAME_TYPES_BY_SHORT_FICS_NAME["B"] will be fixed in FICSConnections.py
+
+
 def type_to_display_text(typename):
     if "loaded from" in typename.lower():
         typename = typename.split()[-1]
@@ -210,7 +234,8 @@ TYPE_ADMINISTRATOR, TYPE_BLINDFOLD, TYPE_COMPUTER, \
     TYPE_SERVICE_REPRESENTATIVE, TYPE_TOURNAMENT_DIRECTOR, TYPE_MAMER_MANAGER, \
     TYPE_GRAND_MASTER, TYPE_INTERNATIONAL_MASTER, TYPE_FIDE_MASTER, \
     TYPE_WOMAN_GRAND_MASTER, TYPE_WOMAN_INTERNATIONAL_MASTER, TYPE_WOMAN_FIDE_MASTER,\
-    TYPE_DUMMY_ACCOUNT, TYPE_CANDIDATE_MASTER, TYPE_FIDE_ARBEITER, TYPE_NATIONAL_MASTER = range(19)
+    TYPE_DUMMY_ACCOUNT, TYPE_CANDIDATE_MASTER, TYPE_FIDE_ARBEITER, TYPE_NATIONAL_MASTER, \
+    TYPE_DISPLAY_MASTER = range(20)
 
 TITLE_TYPE_DISPLAY_TEXTS = (_("Administrator"),
                             _("Blindfold Account"),
@@ -227,12 +252,17 @@ TITLE_TYPE_DISPLAY_TEXTS = (_("Administrator"),
                             _("Woman Grand Master"),
                             _("Woman International Master"),
                             _("Woman FIDE Master"),
-                            _("Dummy Account"), )
+                            _("Dummy Account"),
+                            _("Candidate Master"),
+                            _("FIDE Arbeiter"),
+                            _("National Master"),
+                            _("Display Master"),
+                            )
 
 TITLE_TYPE_DISPLAY_TEXTS_SHORT = (
     _("*"), _("B"), _("C"), _("T"), _("U"), _("CA"), _("SR"), _("TD"), _("TM"),
     _("GM"), _("IM"), _("FM"), _("WGM"), _("WIM"), _("WFM"), _("D"), _("H"),
-    _("CM"), _("FA"), _("NM"))
+    _("CM"), _("FA"), _("NM"), _("DM"))
 
 TITLES = {  # From FICS 'help who'
     "*": TYPE_ADMINISTRATOR,
@@ -255,6 +285,7 @@ TITLES = {  # From FICS 'help who'
     "CM": TYPE_CANDIDATE_MASTER,
     "FA": TYPE_FIDE_ARBEITER,
     "NM": TYPE_NATIONAL_MASTER,
+    "DM": TYPE_DISPLAY_MASTER,
 }
 
 HEX_TO_TITLE = {
