@@ -1,5 +1,4 @@
 import re
-from threading import Timer
 
 from gi.repository import GObject
 
@@ -34,6 +33,14 @@ class HelperManager(GObject.GObject):
         self.connection = connection
 
         # TODO: Examined games
+        # 25 (Exam.    0 Friar          0 Friar     ) [ uu  0   0] W:  1
+        # 28 ++++ TryMe       1737 Jack       [ su 30  20]  22:27 - 23:17 (29-30) W: 16
+        # 2 2274 OldManII    ++++ Peshkin    [ bu  2  12]   2:34 -  1:47 (39-39) B:  3
+        # 29 1622 Vman        1609 PopKid     [ sr 10  10]   1:14 -  5:10 (21-22) B: 18
+        # 32 1880 Raskapov    1859 RoboDweeb  [ br  2  12]   1:04 -  1:26 ( 9-10) B: 34
+        # 1 1878 Roberto     1881 baraka     [psr 45  30]  30:35 - 34:24 (22-22) W: 21
+        #
+        # 6 games displayed (of 23 in progress)
         self.helperconn.expect_fromto(
             self.on_game_list,
             "(\d+) %s (\w+)\s+%s (\w+)\s+\[(p| )(%s)(u|r)\s*(\d+)\s+(\d+)\]\s*(\d:)?(\d+):(\d+)\s*-\s*(\d:)?(\d+):(\d+) \(\s*(\d+)-\s*(\d+)\) (W|B):\s*(\d+)"
@@ -70,6 +77,7 @@ class HelperManager(GObject.GObject):
             % (names, ratings, ratings, ratings, ratings, ratings),
             "is now available for matches.")
 
+        # FICS game types
         # b: blitz      l: lightning   u: untimed      e: examined game
         # s: standard   w: wild        x: atomic       z: crazyhouse
         # B: Bughouse   L: losers      S: Suicide
@@ -122,8 +130,7 @@ class HelperManager(GObject.GObject):
                 if player.game != game:
                     player.game = game
                 rating = parseRating(rating)
-                if gametype.rating_type in player.ratings and \
-                        player.ratings[gametype.rating_type] != rating:
+                if player.ratings[gametype.rating_type] != rating:
                     player.ratings[gametype.rating_type] = rating
                     player.emit("ratings_changed", gametype.rating_type, player)
             game = self.connection.games.get(game, emit=False)
@@ -307,8 +314,8 @@ class HelperManager(GObject.GObject):
             player.titles |= titles
 
         for rating_type, rating in ((TYPE_BLITZ, blitz), (TYPE_STANDARD, std),
-                              (TYPE_LIGHTNING, light), (TYPE_WILD, wild),
-                              (TYPE_BUGHOUSE, bughouse)):
+                                    (TYPE_LIGHTNING, light), (TYPE_WILD, wild),
+                                    (TYPE_BUGHOUSE, bughouse)):
             rating = parseRating(rating)
             if player.ratings[rating_type] != rating:
                 player.ratings[rating_type] = rating
