@@ -1,15 +1,15 @@
-from __future__ import absolute_import
-from . import gamewidget
 from pychess.Utils.const import BLACK, WHITE
+from pychess.perspectives import perspective_manager
 
 firstRun = True
 
 
 def run(widgets):
-    gamemodel = gamewidget.cur_gmwidg().gamemodel
+    persp = perspective_manager.get_perspective("games")
+    gamemodel = persp.cur_gmwidg().gamemodel
     widgets["event_entry"].set_text(gamemodel.tags["Event"])
     widgets["site_entry"].set_text(gamemodel.tags["Site"])
-    widgets["round_spinbutton"].set_value(float(gamemodel.tags["Round"]))
+    widgets["round_entry"].set_text(str(gamemodel.tags["Round"]))
     widgets["white_entry"].set_text(gamemodel.tags["White"])
     widgets["black_entry"].set_text(gamemodel.tags["Black"])
 
@@ -33,10 +33,11 @@ def initialize(widgets):
         return True
 
     def accept_new_properties(button, *args):
-        gamemodel = gamewidget.cur_gmwidg().gamemodel
+        persp = perspective_manager.get_perspective("games")
+        gamemodel = persp.cur_gmwidg().gamemodel
         gamemodel.tags["Event"] = widgets["event_entry"].get_text()
         gamemodel.tags["Site"] = widgets["site_entry"].get_text()
-        gamemodel.tags["Round"] = int(widgets["round_spinbutton"].get_value())
+        gamemodel.tags["Round"] = widgets["round_entry"].get_text()
         gamemodel.tags["White"] = widgets["white_entry"].get_text()
         gamemodel.tags["Black"] = widgets["black_entry"].get_text()
         gamemodel.tags["Year"] = widgets["game_info_calendar"].get_date()[0]
