@@ -1,12 +1,12 @@
 """ This module intends to work as glue between the gamemodel and the gamewidget
     taking care of stuff that is neither very offscreen nor very onscreen
     like bringing up dialogs and """
-from __future__ import absolute_import
 
+import asyncio
 import math
 from collections import defaultdict
 
-from gi.repository import GLib, Gtk
+from gi.repository import Gtk
 
 from pychess.ic.FICSObjects import make_sensitive_if_available, make_sensitive_if_playing
 from pychess.ic.ICGameModel import ICGameModel
@@ -17,13 +17,12 @@ from pychess.Utils.const import WAITING_TO_START, WHITE, BLACK, WHITEWON, \
     DRAW_OFFER, PAUSE_OFFER, RESUME_OFFER, HURRY_ACTION, FLAG_CALL
 from pychess.Utils.repr import reprResult_long, reprReason_long
 from pychess.System import conf
-from pychess.System.idle_add import idle_add
 from pychess.System.Log import log
 from pychess.widgets import preferencesDialog
 from pychess.widgets.InfoBar import InfoBarMessage, InfoBarMessageButton
-from pychess.widgets import InfoBar
-
-from .gamewidget import getWidgets, key2gmwidg
+from pychess.widgets import InfoBar, mainwindow
+from pychess.widgets.gamewidget import getWidgets
+from pychess.perspectives import perspective_manager
 
 
 class GameNanny(object):
@@ -68,23 +67,22 @@ class GameNanny(object):
                 button.set_property("tooltip-text", "")
 
         if gamewidget.game_ended_message:
-            GLib.idle_add(disable_buttons)
+            disable_buttons
 
     # ===============================================================================
     # Gamewidget signals
     # ===============================================================================
 
-    @idle_add
     def on_gmwidg_closed(self, gmwidg):
-        if len(key2gmwidg) == 1:
-            getWidgets()['window1'].set_title('%s - PyChess' % _('Welcome'))
+        perspective = perspective_manager.get_perspective("games")
+        if len(perspective.key2gmwidg) == 1:
+            getWidgets()['main_window'].set_title('%s - PyChess' % _('Welcome'))
         return False
 
-    @idle_add
     def on_gmwidg_title_changed(self, gmwidg, new_title):
         # log.debug("gamenanny.on_gmwidg_title_changed: starting %s" % repr(gmwidg))
         if gmwidg.isInFront():
-            getWidgets()['window1'].set_title('%s - PyChess' % new_title)
+            getWidgets()['main_window'].set_title('%s - PyChess' % new_title)
         # log.debug("gamenanny.on_gmwidg_title_changed: returning")
         return False
 
@@ -92,7 +90,6 @@ class GameNanny(object):
     # Gamemodel signals
     # ===============================================================================
 
-    @idle_add
     def game_ended(self, gamemodel, reason, gmwidg):
         log.debug("gamenanny.game_ended: reason=%s gmwidg=%s\ngamemodel=%s" %
                   (reason, gmwidg, gamemodel))
@@ -157,7 +154,7 @@ class GameNanny(object):
 
             def callback(infobar, response, message, gamemodel=gamemodel):
                 if response == 1:
-                    # newGameDialog uses ionest uses gamenanny uses newGameDialog...
+                    # newGameDialog uses perspectives.games uses gamenanny uses newGameDialog...
                     from pychess.widgets.newGameDialog import createRematch
                     createRematch(gamemodel)
                 elif response == 2:
@@ -183,7 +180,8 @@ class GameNanny(object):
         message.callback = callback
         gmwidg.game_ended_message = message
 
-        if len(key2gmwidg) > 0:
+        perspective = perspective_manager.get_perspective("games")
+        if len(perspective.key2gmwidg) > 0:
             gmwidg.replaceMessages(message)
 
         if reason == WHITE_ENGINE_DIED:
@@ -193,8 +191,8 @@ class GameNanny(object):
 
         if (isinstance(gamemodel, ICGameModel) and not gamemodel.isObservationGame()) or \
                 gamemodel.isEngine2EngineGame():
-            gamemodel.restart_analyzer(HINT)
-            gamemodel.restart_analyzer(SPY)
+            asyncio.async(gamemodel.restart_analyzer(HINT))
+            asyncio.async(gamemodel.restart_analyzer(SPY))
             if not conf.get("hint_mode", False):
                 gamemodel.pause_analyzer(HINT)
             if not conf.get("spy_mode", False):
@@ -222,11 +220,9 @@ class GameNanny(object):
                     "offer", self.offer_callback, gamemodel, gmwidg)
 
         # Start analyzers if any
-        if not (isinstance(gamemodel, ICGameModel) and
-                gamemodel.isObservationGame() is False) and \
-           not gamemodel.isEngine2EngineGame():
-            gamemodel.start_analyzer(HINT)
-            gamemodel.start_analyzer(SPY)
+        if not gamemodel.isEngine2EngineGame():
+            asyncio.async(gamemodel.start_analyzer(HINT))
+            asyncio.async(gamemodel.start_analyzer(SPY))
             if not conf.get("hint_mode", False):
                 gamemodel.pause_analyzer(HINT)
             if not conf.get("spy_mode", False):
@@ -237,7 +233,6 @@ class GameNanny(object):
     # Player signals
     # ===============================================================================
 
-    @idle_add
     def offer_callback(self, player, offer, gamemodel, gmwidg):
         if gamemodel.status != RUNNING:
             # If the offer has already been handled by Gamemodel and the game was
@@ -279,7 +274,7 @@ class GameNanny(object):
 
     def engineDead(self, engine, gmwidg):
         gmwidg.bringToFront()
-        dialog = Gtk.MessageDialog(type=Gtk.MessageType.ERROR,
+        dialog = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.ERROR,
                                    buttons=Gtk.ButtonsType.OK)
         dialog.set_markup(_("<big><b>Engine, %s, has died</b></big>") % repr(engine))
         dialog.format_secondary_text(_(
