@@ -1,31 +1,24 @@
 """ This module handles the tabbed layout in PyChess """
-from __future__ import absolute_import
 
-import imp
-import os
 import sys
-import traceback
-import threading
 from collections import defaultdict
-from threading import currentThread
+from io import StringIO
 
-from gi.repository import Gdk, Gtk, GLib, GObject
+from gi.repository import Gdk, Gtk, GObject
 
-from pychess.compat import StringIO
+import pychess
 from .BoardControl import BoardControl
 from .ChessClock import ChessClock
 from .MenuItemsDict import MenuItemsDict
 from pychess.Savers import pgn, fen
-from pychess.System import conf, prefix
+from pychess.System import conf
 
 from pychess.System.Log import log
-from pychess.System.idle_add import idle_add
-from pychess.System.prefix import addUserConfigPrefix
-from pychess.Utils.GameModel import GameModel
-from pychess.Utils.IconLoader import load_icon, get_pixbuf
+from pychess.Utils.IconLoader import load_icon
 from pychess.Utils.const import REMOTE, UNFINISHED_STATES, PAUSED, RUNNING, LOCAL, \
     WHITE, BLACK, ACTION_MENU_ITEMS, DRAW, UNDOABLE_STATES, HINT, SPY, WHITEWON, \
     MENU_ITEMS, BLACKWON, DROP, FAN_PIECES
+from pychess.Utils.GameModel import GameModel
 from pychess.Utils.Move import listToMoves
 from pychess.Utils.lutils import lmove
 from pychess.Utils.lutils.lmove import ParsingError
@@ -34,8 +27,7 @@ from pychess.ic import get_infobarmessage_content, get_infobarmessage_content2
 from pychess.ic.FICSObjects import get_player_tooltip_text
 from pychess.ic.ICGameModel import ICGameModel
 from pychess.widgets.InfoBar import InfoBarNotebook, InfoBarMessage, InfoBarMessageButton
-from .pydock.PyDockTop import PyDockTop
-from .pydock.__init__ import CENTER, EAST, SOUTH
+from pychess.perspectives import perspective_manager
 
 # ###############################################################################
 # Initialize module constants, and a few worker functions                       #
@@ -51,19 +43,6 @@ def createAlignment(top, right, bottom, left):
     return align
 
 
-def cleanNotebook(name=None):
-    def customGetTabLabelText(child):
-        return name
-
-    notebook = Gtk.Notebook()
-    if name is not None:
-        notebook.set_name(name)
-    notebook.get_tab_label_text = customGetTabLabelText
-    notebook.set_show_tabs(False)
-    notebook.set_show_border(False)
-    return notebook
-
-
 def createImage(pixbuf):
     image = Gtk.Image()
     image.set_from_pixbuf(pixbuf)
@@ -74,18 +53,6 @@ light_on = load_icon(16, "stock_3d-light-on", "weather-clear")
 light_off = load_icon(16, "stock_3d-light-off", "weather-clear-night")
 gtk_close = load_icon(16, "gtk-close", "window-close")
 
-media_previous = load_icon(24, "gtk-media-previous-ltr", "media-skip-backward")
-media_rewind = load_icon(24, "gtk-media-rewind-ltr", "media-seek-backward")
-media_forward = load_icon(24, "gtk-media-forward-ltr", "media-seek-forward")
-media_next = load_icon(24, "gtk-media-next-ltr", "media-skip-forward")
-media_eject = load_icon(24, "player-eject", "media-eject")
-
-path = prefix.addDataPrefix("sidepanel")
-postfix = "Panel.py"
-files = [f[:-3] for f in os.listdir(path) if f.endswith(postfix)]
-sidePanels = [imp.load_module(f, *imp.find_module(f, [path])) for f in files]
-
-dockLocation = addUserConfigPrefix("pydock.xml")
 
 # ###############################################################################
 # Initialize module variables                                                  #
@@ -97,41 +64,13 @@ widgets = None
 def setWidgets(w):
     global widgets
     widgets = w
+    pychess.widgets.main_window = widgets["main_window"]
 
 
 def getWidgets():
     return widgets
 
 
-key2gmwidg = {}
-key2cid = {}
-
-notebooks = {}
-docks = {}
-
-
-def get_clean_notebooks(mainvbox):
-    """ This function is just a workaround to "Segfault on second match" issue
-        https://github.com/pychess/pychess/issues/1354
-        I think it's caused by Gtk+ 3.20 CSS theming changes
-    """
-    global notebooks, docks
-    if len(mainvbox.get_children()) == 3:
-        return notebooks
-    else:
-        notebooks = {"board": cleanNotebook("board"),
-                     "buttons": cleanNotebook("buttons"),
-                     "messageArea": cleanNotebook("messageArea")}
-        for panel in sidePanels:
-            notebooks[panel.__name__] = cleanNotebook(panel.__name__)
-        docks = {"board": (Gtk.Label(label="Board"), notebooks["board"])}
-        return notebooks
-
-# ###############################################################################
-# The holder class for tab releated widgets                                    #
-# ###############################################################################
-
-
 class GameWidget(GObject.GObject):
 
     __gsignals__ = {
@@ -140,9 +79,10 @@ class GameWidget(GObject.GObject):
         'closed': (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
-    def __init__(self, gamemodel):
+    def __init__(self, gamemodel, perspective):
         GObject.GObject.__init__(self)
         self.gamemodel = gamemodel
+        self.perspective = perspective
         self.cids = {}
         self.closed = False
 
@@ -196,20 +136,6 @@ class GameWidget(GObject.GObject):
                 self.gamemodel.connection.bm.connect("opp_not_out_of_time", self.opp_not_out_of_time))
         self.cids[self.board.view] = self.board.view.connect("shownChanged", self.shownChanged)
 
-        def do_load_panels(event):
-            # self.panels = [panel.Sidepanel().load(self) if panel.__name__ == "bookPanel" else Gtk.ScrolledWindow() for panel in sidePanels]
-            self.panels = [panel.Sidepanel().load(self) for panel in sidePanels]
-            if event is not None:
-                event.set()
-
-        thread = currentThread()
-        if thread.name == "MainThread":
-            do_load_panels(None)
-        else:
-            event = threading.Event()
-            GLib.idle_add(do_load_panels, event)
-            event.wait()
-
         if isinstance(self.gamemodel, ICGameModel):
             self.gamemodel.gmwidg_ready.set()
 
@@ -246,7 +172,6 @@ class GameWidget(GObject.GObject):
         else:
             self.game_info_label.set_text("")
 
-    @idle_add
     def infront(self):
         for menuitem in self.menuitems:
             self.menuitems[menuitem].update()
@@ -260,7 +185,7 @@ class GameWidget(GObject.GObject):
                 getWidgets()[widget].set_property('sensitive', True)
 
         # Change window title
-        getWidgets()['window1'].set_title('%s - PyChess' % self.display_text)
+        getWidgets()['main_window'].set_title('%s - PyChess' % self.display_text)
 
     def _update_menu_abort(self):
         if self.gamemodel.hasEnginePlayer():
@@ -442,7 +367,7 @@ class GameWidget(GObject.GObject):
         self._update_menu_undo()
         if isinstance(gamemodel,
                       ICGameModel):  # on FICS game board change update allob
-            if gamemodel.connection is not None:
+            if gamemodel.connection is not None and not gamemodel.connection.ICC:
                 allob = 'allob ' + str(gamemodel.ficsgame.gameno)
                 gamemodel.connection.client.run_command(allob)
 
@@ -598,17 +523,15 @@ class GameWidget(GObject.GObject):
         log.debug("GameWidget.name_changed: starting %s" % repr(player))
         color = self.gamemodel.color(player)
 
-        def do_name_changed():
-            if self.gamemodel is None:
-                return
-            self.player_name_labels[color].set_text(self.player_display_text(
-                color=color))
-            if isinstance(self.gamemodel, ICGameModel) and \
-                    player.__type__ == REMOTE:
-                self.player_name_labels[color].set_tooltip_text(
-                    get_player_tooltip_text(self.gamemodel.ficsplayers[color],
-                                            show_status=False))
-        GLib.idle_add(do_name_changed)
+        if self.gamemodel is None:
+            return
+        self.player_name_labels[color].set_text(self.player_display_text(
+            color=color))
+        if isinstance(self.gamemodel, ICGameModel) and \
+                player.__type__ == REMOTE:
+            self.player_name_labels[color].set_tooltip_text(
+                get_player_tooltip_text(self.gamemodel.ficsplayers[color],
+                                        show_status=False))
 
         self.emit('title_changed', self.display_text)
         log.debug("GameWidget.name_changed: returning")
@@ -637,7 +560,6 @@ class GameWidget(GObject.GObject):
                 self.menuitems["call_flag"].sensitive = True
                 break
 
-    @idle_add
     def player_lagged(self, bm, player):
         if player in self.gamemodel.ficsplayers:
             content = get_infobarmessage_content(
@@ -655,7 +577,6 @@ class GameWidget(GObject.GObject):
             self.showMessage(message)
         return False
 
-    @idle_add
     def opp_not_out_of_time(self, bm):
         if self.gamemodel is not None and self.gamemodel.remote_player.time <= 0:
             content = get_infobarmessage_content2(
@@ -731,57 +652,71 @@ class GameWidget(GObject.GObject):
         return boardvbox, board, infobar, cclock
 
     def initButtons(self, board):
-        def tip(widget, x, y, keyboard_mode, tooltip, text):
-            label = Gtk.Label(label=text)
-            tooltip.set_custom(label)
-            label.show()
-            return True
-
         align = createAlignment(4, 0, 4, 0)
-        page_hbox = Gtk.HBox()
-        startbut = Gtk.Button()
-        startbut.add(createImage(media_previous))
-        startbut.set_relief(Gtk.ReliefStyle.NONE)
-        startbut.props.has_tooltip = True
-        startbut.connect("query-tooltip", tip, _("Jump to initial position"))
-        backbut = Gtk.Button()
-        backbut.add(createImage(media_rewind))
-        backbut.set_relief(Gtk.ReliefStyle.NONE)
-        backbut.props.has_tooltip = True
-        backbut.connect("query-tooltip", tip, _("Step back one move"))
-        mainbut = Gtk.Button()
-        mainbut.add(createImage(media_eject))
-        mainbut.set_relief(Gtk.ReliefStyle.NONE)
-        mainbut.props.has_tooltip = True
-        mainbut.connect("query-tooltip", tip, _("Go back to the main line"))
+        toolbar = Gtk.Toolbar()
 
-        forwbut = Gtk.Button()
-        forwbut.add(createImage(media_forward))
-        forwbut.set_relief(Gtk.ReliefStyle.NONE)
-        forwbut.props.has_tooltip = True
-        forwbut.connect("query-tooltip", tip, _("Step forward one move"))
-        endbut = Gtk.Button()
-        endbut.add(createImage(media_next))
-        endbut.set_relief(Gtk.ReliefStyle.NONE)
-        endbut.props.has_tooltip = True
-        endbut.connect("query-tooltip", tip, _("Jump to latest position"))
+        firstButton = Gtk.ToolButton(Gtk.STOCK_MEDIA_PREVIOUS)
+        firstButton.set_tooltip_text(_("Jump to initial position"))
+        toolbar.insert(firstButton, -1)
+
+        prevButton = Gtk.ToolButton(Gtk.STOCK_MEDIA_REWIND)
+        prevButton.set_tooltip_text(_("Step back one move"))
+        toolbar.insert(prevButton, -1)
+
+        mainButton = Gtk.ToolButton(Gtk.STOCK_REDO)
+        mainButton.set_tooltip_text(_("Go back to the main line"))
+        toolbar.insert(mainButton, -1)
+
+        nextButton = Gtk.ToolButton(Gtk.STOCK_MEDIA_FORWARD)
+        nextButton.set_tooltip_text(_("Step forward one move"))
+        toolbar.insert(nextButton, -1)
+
+        lastButton = Gtk.ToolButton(Gtk.STOCK_MEDIA_NEXT)
+        lastButton.set_tooltip_text(_("Jump to latest position"))
+        toolbar.insert(lastButton, -1)
+
+        filterButton = Gtk.ToggleToolButton(Gtk.STOCK_FIND)
+        filterButton.set_tooltip_text(_("Find postion in current database"))
+        toolbar.insert(filterButton, -1)
 
         def on_clicked(button, func):
             func()
-        self.cids[startbut] = startbut.connect("clicked", on_clicked, self.board.view.showFirst)
-        self.cids[backbut] = backbut.connect("clicked", on_clicked, self.board.view.showPrev)
-        self.cids[mainbut] = mainbut.connect("clicked", on_clicked, self.board.view.backToMainLine)
-        self.cids[forwbut] = forwbut.connect("clicked", on_clicked, self.board.view.showNext)
-        self.cids[endbut] = endbut.connect("clicked", on_clicked, self.board.view.showLast)
-        page_hbox.pack_start(startbut, True, True, 0)
-        page_hbox.pack_start(backbut, True, True, 0)
-        page_hbox.pack_start(mainbut, True, True, 0)
-        page_hbox.pack_start(forwbut, True, True, 0)
-        page_hbox.pack_start(endbut, True, True, 0)
-        align.add(page_hbox)
+
+        self.cids[firstButton] = firstButton.connect("clicked", on_clicked, self.board.view.showFirst)
+        self.cids[prevButton] = prevButton.connect("clicked", on_clicked, self.board.view.showPrev)
+        self.cids[mainButton] = mainButton.connect("clicked", on_clicked, self.board.view.backToMainLine)
+        self.cids[nextButton] = nextButton.connect("clicked", on_clicked, self.board.view.showNext)
+        self.cids[lastButton] = lastButton.connect("clicked", on_clicked, self.board.view.showLast)
+        self.cids[filterButton] = filterButton.connect("clicked", on_clicked, self.find_in_database)
+
+        tool_box = Gtk.Box()
+        tool_box.pack_start(toolbar, True, True, 0)
+
+        align.add(tool_box)
         return align
 
-    @idle_add
+    def find_in_database(self):
+        persp = perspective_manager.get_perspective("database")
+        if persp.chessfile is None:
+            return
+
+        view = self.board.view
+        shown_board = self.gamemodel.getBoardAtPly(view.shown, view.shown_variation_idx)
+        fen = shown_board.asFen()
+
+        result = persp.chessfile.get_book_moves(fen)
+        if len(result) == 0:
+            dialogue = Gtk.MessageDialog(pychess.widgets.mainwindow(), type=Gtk.MessageType.WARNING,
+                                         buttons=Gtk.ButtonsType.OK,
+                                         message_format=_("Position not in current database"))
+            dialogue.run()
+            dialogue.destroy()
+        else:
+            persp.chessfile.set_fen_filter(fen)
+            persp.gamelist.ply = view.shown
+            persp.gamelist.load_games()
+            perspective_manager.activate_perspective("database")
+
     def light_on_off(self, on):
         child = self.tabcontent.get_child()
         if child:
@@ -812,24 +747,24 @@ class GameWidget(GObject.GObject):
                   self.gamemodel.players)
 
     def bringToFront(self):
-        getheadbook().set_current_page(self.getPageNumber())
+        self.perspective.getheadbook().set_current_page(self.getPageNumber())
 
     def isInFront(self):
-        if not getheadbook():
+        if not self.perspective.getheadbook():
             return False
-        return getheadbook().get_current_page() == self.getPageNumber()
+        return self.perspective.getheadbook().get_current_page() == self.getPageNumber()
 
     def getPageNumber(self):
-        return getheadbook().page_num(self.notebookKey)
+        return self.perspective.getheadbook().page_num(self.notebookKey)
 
     def infobar_hidden(self, infobar):
-        if self == cur_gmwidg():
-            notebooks["messageArea"].hide()
+        if self == self.perspective.cur_gmwidg():
+            self.perspective.notebooks["messageArea"].hide()
 
     def showMessage(self, message):
         self.infobar.push_message(message)
-        if self == cur_gmwidg():
-            notebooks["messageArea"].show()
+        if self == self.perspective.cur_gmwidg():
+            self.perspective.notebooks["messageArea"].show()
 
     def replaceMessages(self, message):
         """ Replace all messages with message """
@@ -839,8 +774,8 @@ class GameWidget(GObject.GObject):
 
     def clearMessages(self):
         self.infobar.clear_messages()
-        if self == cur_gmwidg():
-            notebooks["messageArea"].hide()
+        if self == self.perspective.cur_gmwidg():
+            self.perspective.notebooks["messageArea"].hide()
 
     def copy_pgn(self):
         output = StringIO()
@@ -853,362 +788,6 @@ class GameWidget(GObject.GObject):
         clipboard.set_text(
             fen.save(output, self.gamemodel, self.board.view.shown), -1)
 
-# ###############################################################################
-# Main handling of gamewidgets                                                 #
-# ###############################################################################
-
-
-def splitit(widget):
-    if not hasattr(widget, 'get_children'):
-        return
-    for child in widget.get_children():
-        splitit(child)
-        widget.remove(child)
-
-
-def delGameWidget(gmwidg):
-    """ Remove the widget from the GUI after the game has been terminated """
-    global dock, dockAlign
-    log.debug("gamewidget.delGameWidget: starting %s" % repr(gmwidg))
-    gmwidg.closed = True
-    gmwidg.emit("closed")
-
-    called_from_preferences = False
-    # wl = Gtk.window_list_toplevels()
-    window_list = Gtk.Window.list_toplevels()
-    for window in window_list:
-        if window.is_active() and window == widgets["preferences"]:
-            called_from_preferences = True
-            break
-
-    if gmwidg.notebookKey in key2gmwidg:
-        del key2gmwidg[gmwidg.notebookKey]
-    pageNum = gmwidg.getPageNumber()
-    headbook = getheadbook()
-
-    if gmwidg.notebookKey in key2cid:
-        headbook.disconnect(key2cid[gmwidg.notebookKey])
-        del key2cid[gmwidg.notebookKey]
-
-    headbook.remove_page(pageNum)
-    for notebook in notebooks.values():
-        notebook.remove_page(pageNum)
-
-    if headbook.get_n_pages() == 1 and conf.get("hideTabs", False):
-        show_tabs(False)
-
-    if headbook.get_n_pages() == 0:
-        dock = None
-        dockAlign = None
-        mainvbox = widgets["mainvbox"]
-        centerVBox = mainvbox.get_children()[2]
-        for child in centerVBox.get_children():
-            centerVBox.remove(child)
-        mainvbox.remove(centerVBox)
-        mainvbox.remove(mainvbox.get_children()[1])
-
-        mainvbox.pack_end(background, True, True, 0)
-        background.show()
-
-        if not called_from_preferences:
-            # If the last (but not the designGW) gmwidg was closed
-            # and we are FICS-ing, present the FICS lounge
-            from pychess.ic.ICLogon import dialog
-            try:
-                dialog.lounge.present()
-            except AttributeError:
-                pass
-    gmwidg._del()
-
-
-def _ensureReadForGameWidgets():
-    mainvbox = widgets["mainvbox"]
-    if len(mainvbox.get_children()) == 3:
-        return
-    global background, notebooks
-    notebooks = get_clean_notebooks(mainvbox)
-    background = widgets["mainvbox"].get_children()[1]
-    mainvbox.remove(background)
-
-    # Initing headbook
-
-    align = createAlignment(4, 4, 0, 4)
-    align.set_property("yscale", 0)
-    headbook = Gtk.Notebook()
-    headbook.set_scrollable(True)
-    align.add(headbook)
-    mainvbox.pack_start(align, False, True, 0)
-    show_tabs(not conf.get("hideTabs", False))
-
-    # Initing center
-
-    centerVBox = Gtk.VBox()
-
-    # The dock
-
-    global dock, dockAlign
-    dock = PyDockTop("main")
-    dockAlign = createAlignment(4, 4, 0, 4)
-    dockAlign.add(dock)
-    centerVBox.pack_start(dockAlign, True, True, 0)
-    dockAlign.show()
-    dock.show()
-
-    for panel in sidePanels:
-        hbox = Gtk.HBox()
-        pixbuf = get_pixbuf(panel.__icon__, 16)
-        icon = Gtk.Image.new_from_pixbuf(pixbuf)
-        label = Gtk.Label(label=panel.__title__)
-        label.set_size_request(0, 0)
-        label.set_alignment(0, 1)
-        hbox.pack_start(icon, False, False, 0)
-        hbox.pack_start(label, True, True, 0)
-        hbox.set_spacing(2)
-        hbox.show_all()
-
-        def cb(widget, x, y, keyboard_mode, tooltip, title, desc, filename):
-            table = Gtk.Table(2, 2)
-            table.set_row_spacings(2)
-            table.set_col_spacings(6)
-            table.set_border_width(4)
-            pixbuf = get_pixbuf(filename, 56)
-            image = Gtk.Image.new_from_pixbuf(pixbuf)
-            image.set_alignment(0, 0)
-            table.attach(image, 0, 1, 0, 2)
-            titleLabel = Gtk.Label()
-            titleLabel.set_markup("<b>%s</b>" % title)
-            titleLabel.set_alignment(0, 0)
-            table.attach(titleLabel, 1, 2, 0, 1)
-            descLabel = Gtk.Label(label=desc)
-            descLabel.props.wrap = True
-            table.attach(descLabel, 1, 2, 1, 2)
-            tooltip.set_custom(table)
-            table.show_all()
-            return True
-
-        hbox.props.has_tooltip = True
-        hbox.connect("query-tooltip", cb, panel.__title__, panel.__desc__,
-                     panel.__icon__)
-
-        docks[panel.__name__] = (hbox, notebooks[panel.__name__])
-
-    if os.path.isfile(dockLocation):
-        try:
-            dock.loadFromXML(dockLocation, docks)
-        except Exception as e:
-            stringio = StringIO()
-            traceback.print_exc(file=stringio)
-            error = stringio.getvalue()
-            log.error("Dock loading error: %s\n%s" % (e, error))
-            msg_dia = Gtk.MessageDialog(widgets["window1"],
-                                        type=Gtk.MessageType.ERROR,
-                                        buttons=Gtk.ButtonsType.CLOSE)
-            msg_dia.set_markup(_(
-                "<b><big>PyChess was unable to load your panel settings</big></b>"))
-            msg_dia.format_secondary_text(_(
-                "Your panel settings have been reset. If this problem repeats, \
-                you should report it to the developers"))
-            msg_dia.run()
-            msg_dia.hide()
-            os.remove(dockLocation)
-            for title, panel in docks.values():
-                title.unparent()
-                panel.unparent()
-
-    if not os.path.isfile(dockLocation):
-        leaf = dock.dock(docks["board"][1],
-                         CENTER,
-                         Gtk.Label(label=docks["board"][0]),
-                         "board")
-        docks["board"][1].show_all()
-        leaf.setDockable(False)
-
-        # S
-        epanel = leaf.dock(docks["bookPanel"][1], SOUTH, docks["bookPanel"][0],
-                           "bookPanel")
-        epanel.default_item_height = 45
-        epanel = epanel.dock(docks["engineOutputPanel"][1], CENTER,
-                             docks["engineOutputPanel"][0],
-                             "engineOutputPanel")
-
-        # NE
-        leaf = leaf.dock(docks["annotationPanel"][1], EAST,
-                         docks["annotationPanel"][0], "annotationPanel")
-        leaf = leaf.dock(docks["historyPanel"][1], CENTER,
-                         docks["historyPanel"][0], "historyPanel")
-        leaf = leaf.dock(docks["scorePanel"][1], CENTER,
-                         docks["scorePanel"][0], "scorePanel")
-
-        # SE
-        leaf = leaf.dock(docks["chatPanel"][1], SOUTH, docks["chatPanel"][0],
-                         "chatPanel")
-        leaf = leaf.dock(docks["commentPanel"][1], CENTER,
-                         docks["commentPanel"][0], "commentPanel")
-
-    def unrealize(dock, notebooks):
-        # unhide the panel before saving so its configuration is saved correctly
-        notebooks["board"].get_parent().get_parent().zoomDown()
-        dock.saveToXML(dockLocation)
-        dock._del()
-
-    dock.connect("unrealize", unrealize, notebooks)
-
-    hbox = Gtk.HBox()
-
-    # Buttons
-    notebooks["buttons"].set_border_width(4)
-    hbox.pack_start(notebooks["buttons"], False, True, 0)
-
-    # The message area
-    # TODO: If you try to fix this first read issue #958 and 1018
-    align = createAlignment(0, 0, 0, 0)
-    # sw = Gtk.ScrolledWindow()
-    # port = Gtk.Viewport()
-    # port.add(notebooks["messageArea"])
-    # sw.add(port)
-    # align.add(sw)
-    align.add(notebooks["messageArea"])
-    hbox.pack_start(align, True, True, 0)
-
-    def ma_switch_page(notebook, gpointer, page_num):
-        notebook.props.visible = notebook.get_nth_page(page_num).\
-            get_child().props.visible
-
-    notebooks["messageArea"].connect("switch-page", ma_switch_page)
-    centerVBox.pack_start(hbox, False, True, 0)
-
-    mainvbox.pack_start(centerVBox, True, True, 0)
-    centerVBox.show_all()
-    mainvbox.show()
-
-    # Connecting headbook to other notebooks
-
-    def hb_switch_page(notebook, gpointer, page_num):
-        for notebook in notebooks.values():
-            notebook.set_current_page(page_num)
-
-#        log.debug("HB_switch ficsgame no. %s , %s " % (key2gmwidg[getheadbook().\
-#           get_nth_page(page_num)].gamemodel.ficsgame.gameno,str(page_num)))
-        gmwidg = key2gmwidg[getheadbook().get_nth_page(page_num)]
-        if isinstance(gmwidg.gamemodel, ICGameModel):
-            primary = "primary " + str(gmwidg.gamemodel.ficsgame.gameno)
-            gmwidg.gamemodel.connection.client.run_command(primary)
-
-    headbook.connect("switch-page", hb_switch_page)
-
-    if hasattr(headbook, "set_tab_reorderable"):
-
-        def page_reordered(widget, child, new_num, headbook):
-            old_num = notebooks["board"].page_num(key2gmwidg[child].boardvbox)
-            if old_num == -1:
-                log.error('Games and labels are out of sync!')
-            else:
-                for notebook in notebooks.values():
-                    notebook.reorder_child(
-                        notebook.get_nth_page(old_num), new_num)
-
-        headbook.connect("page-reordered", page_reordered, headbook)
-
-
-def attachGameWidget(gmwidg):
-    log.debug("attachGameWidget: %s" % gmwidg)
-    _ensureReadForGameWidgets()
-    headbook = getheadbook()
-
-    key2gmwidg[gmwidg.notebookKey] = gmwidg
-
-    headbook.append_page(gmwidg.notebookKey, gmwidg.tabcontent)
-    gmwidg.notebookKey.show_all()
-    # headbook.set_tab_label_packing(gmwidg.notebookKey, True, True, Gtk.PACK_START)
-    if hasattr(headbook, "set_tab_reorderable"):
-        headbook.set_tab_reorderable(gmwidg.notebookKey, True)
-
-    def callback(notebook, gpointer, page_num, gmwidg):
-        if notebook.get_nth_page(page_num) == gmwidg.notebookKey:
-            gmwidg.infront()
-            if gmwidg.gamemodel.players and gmwidg.gamemodel.isObservationGame():
-                gmwidg.light_on_off(False)
-                text = gmwidg.game_info_label.get_text()
-                gmwidg.game_info_label.set_markup(
-                    '<span color="black" weight="bold">%s</span>' % text)
-
-    key2cid[gmwidg.notebookKey] = headbook.connect_after("switch-page", callback, gmwidg)
-    gmwidg.infront()
-
-    align = createAlignment(0, 0, 0, 0)
-    align.show()
-    align.add(gmwidg.infobar)
-    notebooks["messageArea"].append_page(align, None)
-    notebooks["board"].append_page(gmwidg.boardvbox, None)
-    gmwidg.boardvbox.show_all()
-    for panel, instance in zip(sidePanels, gmwidg.panels):
-        notebooks[panel.__name__].append_page(instance, None)
-        instance.show_all()
-    notebooks["buttons"].append_page(gmwidg.stat_hbox, None)
-    gmwidg.stat_hbox.show_all()
-
-    # We should always show tabs if more than one exists
-    if headbook.get_n_pages() == 2:
-        show_tabs(True)
-
-    headbook.set_current_page(-1)
-
-    if headbook.get_n_pages() == 1 and not widgets["show_sidepanels"].get_active():
-        zoomToBoard(True)
-
-
-def cur_gmwidg():
-    headbook = getheadbook()
-    if headbook is None:
-        return None
-    notebookKey = headbook.get_nth_page(headbook.get_current_page())
-    return key2gmwidg[notebookKey]
-
-
-def customGetTabLabelText(child):
-    gmwidg = key2gmwidg[child]
-    return gmwidg.display_text
-
-
-def getheadbook():
-    if len(widgets["mainvbox"].get_children()) == 2:
-        # If the headbook hasn't been added yet
-        return None
-    headbook = widgets["mainvbox"].get_children()[1].get_child()
-    # to help StoryText create widget description
-    headbook.get_tab_label_text = customGetTabLabelText
-    return headbook
-
-
-def zoomToBoard(view_zoomed):
-    if len(widgets["mainvbox"].get_children()) == 2:
-        # If the headbook hasn't been added yet
-        return None
-    if not notebooks["board"].get_parent():
-        return
-    if view_zoomed:
-        notebooks["board"].get_parent().get_parent().zoomUp()
-    else:
-        notebooks["board"].get_parent().get_parent().zoomDown()
-
-
-def show_tabs(show):
-    if show:
-        widgets["mainvbox"].get_children()[1].show_all()
-    else:
-        widgets["mainvbox"].get_children()[1].hide()
-
-
-def tabsCallback(none):
-    head = getheadbook()
-    if not head:
-        return
-    if head.get_n_pages() == 1:
-        show_tabs(not conf.get("hideTabs", False))
-
-
-conf.notify_add("hideTabs", tabsCallback)
-
 # ###############################################################################
 # Handling of the special sidepanels-design-gamewidget used in preferences     #
 # ###############################################################################
@@ -1218,19 +797,22 @@ designGW = None
 
 def showDesignGW():
     global designGW
-    designGW = GameWidget(GameModel())
+    perspective = perspective_manager.get_perspective("games")
+    designGW = GameWidget(GameModel(), perspective)
     if isDesignGWShown():
         return
     getWidgets()["show_sidepanels"].set_active(True)
     getWidgets()["show_sidepanels"].set_sensitive(False)
-    attachGameWidget(designGW)
+    perspective.attachGameWidget(designGW)
 
 
 def hideDesignGW():
     if isDesignGWShown():
-        delGameWidget(designGW)
+        perspective = perspective_manager.get_perspective("games")
+        perspective.delGameWidget(designGW)
     getWidgets()["show_sidepanels"].set_sensitive(True)
 
 
 def isDesignGWShown():
-    return designGW in key2gmwidg.values()
+    perspective = perspective_manager.get_perspective("games")
+    return designGW in perspective.key2gmwidg.values()
