@@ -1,19 +1,15 @@
-from __future__ import print_function
-
+import asyncio
 import os.path
 import gettext
 import locale
 
 from operator import attrgetter
 from itertools import groupby
+from io import StringIO
 
 from gi.repository import Gdk, Gtk, GLib, GObject
-
-from cairo import ImageSurface
-
 from gi.repository import GtkSource
 
-from pychess.compat import StringIO
 from pychess.Utils.IconLoader import load_icon, get_pixbuf
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.SetupModel import SetupModel, SetupPlayer
@@ -32,15 +28,15 @@ from pychess.System import conf
 from pychess.System.prefix import getDataPrefix, isInstalled, addDataPrefix
 from pychess.Players.engineNest import discoverer
 from pychess.Players.Human import Human
-from pychess.widgets import BoardPreview
-from pychess.widgets.ionest import game_handler
-from pychess.widgets import gamewidget
 from pychess.widgets import ImageMenu
+from pychess.widgets import mainwindow
 from pychess.widgets.BoardControl import BoardControl
 from pychess.Savers import fen, pgn
 from pychess.Savers.ChessFile import LoadingError
 from pychess.Variants import variants
 from pychess.Variants.normal import NormalBoard
+from pychess.perspectives import perspective_manager
+from pychess.perspectives.games import enddir
 
 # ===============================================================================
 # We init most dialog icons global to make them accessibly to the
@@ -101,7 +97,7 @@ def createPlayerUIGlobals(discoverer):
 
 discoverer.connect("all_engines_discovered", createPlayerUIGlobals)
 
-COPY, CLEAR, PASTE = 2, 3, 4
+COPY, CLEAR, PASTE, INITIAL = 2, 3, 4, 5
 
 # ===============================================================================
 # GameInitializationMode is the super class of new game dialogs. Dialogs include
@@ -124,7 +120,7 @@ class _GameInitializationMode(object):
     @classmethod
     def _init(cls):
         cls.widgets = uistuff.GladeWidgets("newInOut.glade")
-        cls.widgets["newgamedialog"].set_transient_for(gamewidget.getWidgets()["window1"])
+        cls.widgets["newgamedialog"].set_transient_for(mainwindow())
 
         uistuff.createCombo(cls.widgets["whitePlayerCombobox"],
                             name="whitePlayerCombobox")
@@ -156,13 +152,16 @@ class _GameInitializationMode(object):
 
         cls.__initTimeRadio(
             _("Blitz"), "ngblitz", cls.widgets["blitzRadio"],
-            cls.widgets["configImageBlitz"], 5, 0)
+            cls.widgets["configImageBlitz"], 5, 0, 0)
         cls.__initTimeRadio(
             _("Rapid"), "ngrapid", cls.widgets["rapidRadio"],
-            cls.widgets["configImageRapid"], 15, 5)
+            cls.widgets["configImageRapid"], 15, 5, 0)
         cls.__initTimeRadio(
             _("Normal"), "ngnormal", cls.widgets["normalRadio"],
-            cls.widgets["configImageNormal"], 40, 15)
+            cls.widgets["configImageNormal"], 40, 15, 0)
+        cls.__initTimeRadio(
+            _("Classical"), "ngclassical", cls.widgets["classicalRadio"],
+            cls.widgets["configImageClassical"], 3, 0, 40)
 
         cls.__initVariantRadio("ngvariant1", cls.widgets["playVariant1Radio"],
                                cls.widgets["configImageVariant1"],
@@ -170,7 +169,6 @@ class _GameInitializationMode(object):
         cls.__initVariantRadio("ngvariant2", cls.widgets["playVariant2Radio"],
                                cls.widgets["configImageVariant2"], LOSERSCHESS)
 
-        # @idle_add
         def updateCombos(*args):
             if cls.widgets["playNormalRadio"].get_active():
                 variant = NORMALCHESS
@@ -209,7 +207,7 @@ class _GameInitializationMode(object):
         # in the user comboboxes can be different in different variants
         for key in ("whitePlayerCombobox", "blackPlayerCombobox",
                     "skillSlider1", "skillSlider2", "notimeRadio",
-                    "blitzRadio", "rapidRadio", "normalRadio",
+                    "blitzRadio", "rapidRadio", "normalRadio", "classicalRadio",
                     "playNormalRadio", "playVariant1Radio",
                     "playVariant2Radio"):
             uistuff.keep(cls.widgets[key], key)
@@ -220,16 +218,18 @@ class _GameInitializationMode(object):
 
     @classmethod
     def __initTimeRadio(cls, name, id, radiobutton, configImage, defmin,
-                        defgain):
-        # minSpin = Gtk.SpinButton(Gtk.Adjustment(1,1,240,1))
+                        defgain, defmoves):
         minSpin = Gtk.SpinButton()
         minSpin.set_adjustment(Gtk.Adjustment(1, 0, 240, 1))
-        # gainSpin = Gtk.SpinButton(Gtk.Adjustment(0,-60,60,1))
+        setattr(cls, "%s_min" % id, minSpin)
+        uistuff.keep(minSpin, "%s min" % id, first_value=defmin)
+        movesSpin = Gtk.SpinButton()
+        movesSpin.set_adjustment(Gtk.Adjustment(0, 0, 60, 20))
+        setattr(cls, "%s_moves" % id, movesSpin)
+        uistuff.keep(movesSpin, "%s moves" % id, first_value=defmoves)
         gainSpin = Gtk.SpinButton()
         gainSpin.set_adjustment(Gtk.Adjustment(0, -60, 60, 1))
-        setattr(cls, "%s_min" % id, minSpin)
         setattr(cls, "%s_gain" % id, gainSpin)
-        uistuff.keep(minSpin, "%s min" % id, first_value=defmin)
         uistuff.keep(gainSpin, "%s gain" % id, first_value=defgain)
 
         table = Gtk.Table(2, 2)
@@ -239,10 +239,13 @@ class _GameInitializationMode(object):
         label.props.xalign = 0
         table.attach(label, 0, 1, 0, 1)
         table.attach(minSpin, 1, 2, 0, 1)
-        label = Gtk.Label(label=_("Gain:"))
+        label = Gtk.Label(label=_("Moves:") if defmoves > 0 else _("Gain:"))
         label.props.xalign = 0
         table.attach(label, 0, 1, 1, 2)
-        table.attach(gainSpin, 1, 2, 1, 2)
+        if defmoves > 0:
+            table.attach(movesSpin, 1, 2, 1, 2)
+        else:
+            table.attach(gainSpin, 1, 2, 1, 2)
         alignment = Gtk.Alignment.new(1, 1, 1, 1)
         alignment.set_padding(6, 6, 12, 12)
         alignment.add(table)
@@ -251,6 +254,7 @@ class _GameInitializationMode(object):
         def updateString(spin):
             minutes = minSpin.get_value_as_int()
             gain = gainSpin.get_value_as_int()
+            moves = movesSpin.get_value_as_int()
             if gain > 0:
                 radiobutton.set_label(
                     _("%(name)s %(minutes)d min + %(gain)d sec/move") % {
@@ -265,6 +269,12 @@ class _GameInitializationMode(object):
                         'minutes': minutes,
                         'gain': gain
                     })
+            elif moves > 0:
+                radiobutton.set_label(_("%(name)s %(minutes)d min / %(moves)d moves") % {
+                    'name': name,
+                    'minutes': minutes,
+                    'moves': moves,
+                })
             else:
                 radiobutton.set_label(_("%(name)s %(minutes)d min") % {
                     'name': name,
@@ -272,6 +282,7 @@ class _GameInitializationMode(object):
                 })
 
         minSpin.connect("value-changed", updateString)
+        movesSpin.connect("value-changed", updateString)
         gainSpin.connect("value-changed", updateString)
         updateString(None)
 
@@ -353,14 +364,14 @@ class _GameInitializationMode(object):
                 clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
                 text = clipboard.wait_for_text()
                 # print("got clipboard:", text)
-                if len(text.split()) < 2:
+                if text is None or len(text.split()) < 2:
                     return
                 try:
                     lboard = cls.setupmodel.variant(setup=text).board
                     cls.ini_widgets(lboard.asFen())
                     cls.board_control.emit("action", "SETUP", text)
                 except SyntaxError as e:
-                    d = Gtk.MessageDialog(type=Gtk.MessageType.WARNING,
+                    d = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.WARNING,
                                           buttons=Gtk.ButtonsType.OK,
                                           message_format=e.args[0])
                     if len(e.args) > 1:
@@ -368,6 +379,11 @@ class _GameInitializationMode(object):
                     d.connect("response", lambda d, a: d.hide())
                     d.show()
                 return
+            elif response == INITIAL:
+                lboard = cls.setupmodel.variant(setup=FEN_START).board
+                cls.ini_widgets(lboard.asFen())
+                cls.board_control.emit("action", "SETUP", FEN_START)
+                return
             elif response != Gtk.ResponseType.OK:
                 cls.widgets["newgamedialog"].hide()
                 cls.widgets["newgamedialog"].disconnect(handlerId)
@@ -389,15 +405,23 @@ class _GameInitializationMode(object):
             if cls.widgets["notimeRadio"].get_active():
                 secs = 0
                 incr = 0
+                moves = 0
             elif cls.widgets["blitzRadio"].get_active():
                 secs = cls.ngblitz_min.get_value_as_int() * 60
                 incr = cls.ngblitz_gain.get_value_as_int()
+                moves = 0
             elif cls.widgets["rapidRadio"].get_active():
                 secs = cls.ngrapid_min.get_value_as_int() * 60
                 incr = cls.ngrapid_gain.get_value_as_int()
+                moves = 0
             elif cls.widgets["normalRadio"].get_active():
                 secs = cls.ngnormal_min.get_value_as_int() * 60
                 incr = cls.ngnormal_gain.get_value_as_int()
+                moves = 0
+            elif cls.widgets["classicalRadio"].get_active():
+                secs = cls.ngclassical_min.get_value_as_int() * 60
+                incr = 0
+                moves = cls.ngclassical_moves.get_value_as_int()
 
             # Find players
             player0 = cls.widgets["whitePlayerCombobox"].get_active()
@@ -407,7 +431,7 @@ class _GameInitializationMode(object):
             player1 = playerItems[0].index(playerItems[variant_index][player1])
             diffi1 = int(cls.widgets["skillSlider2"].get_value())
 
-            # Prepare players for ionest
+            # Prepare players
             playertups = []
             for i, playerno, diffi, color in ((0, player0, diffi0, WHITE),
                                               (1, player1, diffi1, BLACK)):
@@ -416,7 +440,7 @@ class _GameInitializationMode(object):
                     name = discoverer.getName(engine)
                     playertups.append((ARTIFICIAL, discoverer.initPlayerEngine,
                                        [engine, color, diffi, variant, secs,
-                                        incr], name))
+                                        incr, moves], name))
                 else:
                     if not playertups or playertups[0][0] != LOCAL:
                         name = conf.get("firstName", _("You"))
@@ -430,7 +454,7 @@ class _GameInitializationMode(object):
                 playertups[0][2].append(True)
                 playertups[1][2].append(True)
 
-            timemodel = TimeModel(secs, incr)
+            timemodel = TimeModel(secs, incr, moves=moves)
             gamemodel = GameModel(timemodel, variant)
 
             if not validate(gamemodel):
@@ -449,7 +473,7 @@ class _GameInitializationMode(object):
         for extension in ("loadsidepanel", "enterGameNotationSidePanel",
                           "setupPositionSidePanel"):
             cls.widgets[extension].hide()
-        for button in ("copy_button", "clear_button", "paste_button"):
+        for button in ("copy_button", "clear_button", "paste_button", "initial_button"):
             cls.widgets[button].hide()
 
 # ###############################################################################
@@ -476,59 +500,10 @@ class NewGameMode(_GameInitializationMode):
 
         cls._hideOthers()
         cls.widgets["newgamedialog"].set_title(_("New Game"))
-        cls._generalRun(game_handler.generalStart, _validate)
-
-# ###############################################################################
-# LoadFileExtension                                                            #
-# ###############################################################################
-
-
-class LoadFileExtension(_GameInitializationMode):
-    @classmethod
-    def _init(cls):
-        opendialog, savedialog, enddir, savecombo, savers = game_handler.getOpenAndSaveDialogs(
-        )
-        cls.filechooserbutton = Gtk.FileChooserButton.new_with_dialog(
-            opendialog)
-        cls.loadSidePanel = BoardPreview.BoardPreview(
-            cls.widgets, cls.filechooserbutton, opendialog, enddir)
-
-    @classmethod
-    def run(cls, uri=None):
-        cls._ensureReady()
-        if cls.widgets["newgamedialog"].props.visible:
-            cls.widgets["newgamedialog"].present()
-            return
-
-        if not uri:
-            res = game_handler.opendialog.run()
-            game_handler.opendialog.hide()
-            if res != Gtk.ResponseType.ACCEPT:
-                return
-        else:
-            if not uri[uri.rfind(".") + 1:] in game_handler.enddir:
-                log.info("Ignoring strange file: %s" % uri)
-                return
-            cls.loadSidePanel.set_filename(uri)
-            cls.filechooserbutton.emit("file-activated")
-
-        cls._hideOthers()
-        cls.widgets["newgamedialog"].set_title(_("Open Game"))
-        cls.widgets["loadsidepanel"].show()
-
-        def _validate(gamemodel):
-            return True
 
         def _callback(gamemodel, p0, p1):
-            if not cls.loadSidePanel.isEmpty():
-                uri = cls.loadSidePanel.get_filename()
-                loader = game_handler.enddir[uri[uri.rfind(".") + 1:]]
-                position = cls.loadSidePanel.getPosition()
-                gameno = cls.loadSidePanel.getGameno()
-                game_handler.generalStart(
-                    gamemodel, p0, p1, (uri, loader, gameno, position))
-            else:
-                game_handler.generalStart(gamemodel, p0, p1)
+            perspective = perspective_manager.get_perspective("games")
+            asyncio.async(perspective.generalStart(gamemodel, p0, p1))
 
         cls._generalRun(_callback, _validate)
 
@@ -657,7 +632,7 @@ class SetupPositionExtension(_GameInitializationMode):
             return
 
         cls._hideOthers()
-        for button in ("copy_button", "clear_button", "paste_button"):
+        for button in ("copy_button", "clear_button", "paste_button", "initial_button"):
             cls.widgets[button].show()
         cls.widgets["newgamedialog"].set_title(_("Setup Position"))
         cls.widgets["setupPositionSidePanel"].show()
@@ -691,7 +666,7 @@ class SetupPositionExtension(_GameInitializationMode):
                 cls.setupmodel.variant(setup=fenstr)
                 return True
             except (AssertionError, LoadingError, SyntaxError) as e:
-                d = Gtk.MessageDialog(type=Gtk.MessageType.WARNING,
+                d = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.WARNING,
                                       buttons=Gtk.ButtonsType.OK,
                                       message_format=e.args[0])
                 if len(e.args) > 1:
@@ -702,8 +677,9 @@ class SetupPositionExtension(_GameInitializationMode):
 
         def _callback(gamemodel, p0, p1):
             text = cls.get_fen()
-            game_handler.generalStart(
-                gamemodel, p0, p1, (StringIO(text), fen, 0, -1))
+            perspective = perspective_manager.get_perspective("games")
+            asyncio.async(perspective.generalStart(
+                gamemodel, p0, p1, (StringIO(text), fen, 0, -1)))
 
         cls._generalRun(_callback, _validate)
 
@@ -819,11 +795,11 @@ class EnterNotationExtension(_GameInitializationMode):
             try:
                 text, loadType = _get_text()
                 chessfile = loadType.load(StringIO(text))
-                chessfile.loadToModel(0, -1, model=gamemodel)
+                chessfile.loadToModel(chessfile.games[0], -1, model=gamemodel)
                 gamemodel.status = WAITING_TO_START
                 return True
             except LoadingError as e:
-                d = Gtk.MessageDialog(type=Gtk.MessageType.WARNING,
+                d = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.WARNING,
                                       buttons=Gtk.ButtonsType.OK,
                                       message_format=e.args[0])
                 d.format_secondary_text(e.args[1])
@@ -833,8 +809,9 @@ class EnterNotationExtension(_GameInitializationMode):
 
         def _callback(gamemodel, p0, p1):
             text, loadType = _get_text()
-            game_handler.generalStart(
-                gamemodel, p0, p1, (StringIO(text), loadType, 0, -1))
+            perspective = perspective_manager.get_perspective("games")
+            asyncio.async(perspective.generalStart(
+                gamemodel, p0, p1, (StringIO(text), loadType, 0, -1)))
 
         cls._generalRun(_callback, _validate)
 
@@ -861,35 +838,6 @@ class ImageButton(Gtk.Button):
         self.add(self.image)
 
 
-class XXXImageButton(Gtk.DrawingArea):
-    def __init__(self, image_paths):
-        GObject.GObject.__init__(self)
-        self.set_events(Gdk.EventMask.EXPOSURE_MASK |
-                        Gdk.EventMask.BUTTON_PRESS_MASK)
-
-        self.connect("draw", self.draw)
-        self.connect("button_press_event", self.buttonPress)
-
-        self.surfaces = [ImageSurface.create_from_png(path)
-                         for path in image_paths]
-        self.current = 0
-
-        width, height = self.surfaces[0].get_width(), self.surfaces[
-            0].get_height()
-        self.size = (0, 0, width, height)
-        self.set_size_request(width, height)
-
-    def draw(self, self_, context):
-        context.set_source_surface(self.surfaces[self.current], 0, 0)
-        context.fill()
-
-    def buttonPress(self, self_, event):
-        if event.button == 1 and event.type == Gdk.EventType.BUTTON_PRESS:
-            self.current = (self.current + 1) % len(self.surfaces)
-            self.window.invalidate_rect(self.size, True)
-            self.window.process_updates(True)
-
-
 def createRematch(gamemodel):
     """ If gamemodel contains only LOCAL or ARTIFICIAL players, this starts a
         new game, based on the info in gamemodel """
@@ -897,9 +845,11 @@ def createRematch(gamemodel):
     if gamemodel.timed:
         secs = gamemodel.timemodel.intervals[0][WHITE]
         gain = gamemodel.timemodel.gain
+        moves = gamemodel.timemodel.moves
     else:
         secs = 0
         gain = 0
+        moves = 0
     newgamemodel = GameModel(TimeModel(secs, gain), variant=gamemodel.variant)
 
     wp = gamemodel.players[WHITE]
@@ -914,7 +864,7 @@ def createRematch(gamemodel):
             engine = discoverer.getEngineByMd5(bp.md5)
             player0tup = (ARTIFICIAL, discoverer.initPlayerEngine,
                           (engine, WHITE, bp.strength, gamemodel.variant, secs,
-                           gain), repr(bp))
+                           gain, moves), repr(bp))
     else:
         player0tup = (bp.__type__, bp.__class__, (WHITE, repr(bp)), repr(bp))
         engine = discoverer.getEngineByMd5(wp.md5)
@@ -922,17 +872,19 @@ def createRematch(gamemodel):
                       (engine, BLACK, wp.strength, gamemodel.variant, secs,
                        gain), repr(wp))
 
-    game_handler.generalStart(newgamemodel, player0tup, player1tup)
+    perspective = perspective_manager.get_perspective("games")
+    asyncio.async(perspective.generalStart(newgamemodel, player0tup, player1tup))
 
 
-def loadFilesAndRun(uris):
-    for uri in uris:
-        uri = splitUri(uri)[1]
-        loader = game_handler.enddir[uri[uri.rfind(".") + 1:]]
-        timemodel = TimeModel(0, 0)
-        gamemodel = GameModel(timemodel)
-        white_name = _("White")
-        black_name = _("Black")
-        p0 = (LOCAL, Human, (WHITE, white_name), white_name)
-        p1 = (LOCAL, Human, (BLACK, black_name), black_name)
-        game_handler.generalStart(gamemodel, p0, p1, (uri, loader, 0, -1))
+def loadFileAndRun(uri):
+    parts = splitUri(uri)
+    uri = parts[1] if len(parts) == 2 else parts[0]
+    loader = enddir[uri[uri.rfind(".") + 1:]]
+    timemodel = TimeModel(0, 0)
+    gamemodel = GameModel(timemodel)
+    white_name = _("White")
+    black_name = _("Black")
+    p0 = (LOCAL, Human, (WHITE, white_name), white_name)
+    p1 = (LOCAL, Human, (BLACK, black_name), black_name)
+    perspective = perspective_manager.get_perspective("games")
+    asyncio.async(perspective.generalStart(gamemodel, p0, p1, (uri, loader, 0, -1)))
