@@ -4,27 +4,28 @@
     the user to setup and use customised sounds or no sounds at all for a variety of in game
     events such as running out of time or piece movement events etc.
 """
-from __future__ import print_function
 
+import asyncio
 import os
 from os import listdir
 from os.path import isdir, isfile, splitext
 
 import sys
 from xml.dom import minidom
+from urllib.request import url2pathname, pathname2url
+from urllib.parse import unquote
 
 from gi.repository import Gtk, GdkPixbuf, Gdk
 
-from pychess.compat import pathname2url, url2pathname, unquote
 from pychess.System.prefix import addDataPrefix, getDataPrefix
-from pychess.System.idle_add import idle_add
 from pychess.System import conf, gstreamer, uistuff
 from pychess.Players.engineNest import discoverer
 from pychess.Utils.const import HINT, SPY, SOUND_MUTE, SOUND_BEEP, SOUND_URI, SOUND_SELECT
 from pychess.Utils.IconLoader import load_icon, get_pixbuf
 from pychess.gfx import Pieces
-from pychess.widgets import Background
+from pychess.widgets import mainwindow
 from pychess.widgets.Background import hexcol, newTheme
+from pychess.perspectives import perspective_manager
 
 firstRun = True
 
@@ -104,16 +105,16 @@ def anal_combo_set_value(combobox, value, show_arrow_check, ana_check,
             index = 0
         combobox.set_active(index)
 
-    from pychess.widgets.ionest import game_handler
     from pychess.widgets.gamewidget import widgets
-    for gmwidg in game_handler.gamewidgets:
+    perspective = perspective_manager.get_perspective("games")
+    for gmwidg in perspective.gamewidgets:
         spectators = gmwidg.gamemodel.spectators
         md5 = engine.get('md5')
 
         if analyzer_type in spectators and \
                 spectators[analyzer_type].md5 != md5:
             gmwidg.gamemodel.remove_analyzer(analyzer_type)
-            gmwidg.gamemodel.start_analyzer(analyzer_type)
+            asyncio.async(gmwidg.gamemodel.start_analyzer(analyzer_type))
             if not widgets[show_arrow_check].get_active():
                 gmwidg.gamemodel.pause_analyzer(analyzer_type)
 
@@ -133,7 +134,7 @@ class HintTab:
         conf.set("opening_file_entry", path)
 
         book_chooser_dialog = Gtk.FileChooserDialog(
-            _("Select book file"), None, Gtk.FileChooserAction.OPEN,
+            _("Select book file"), mainwindow(), Gtk.FileChooserAction.OPEN,
             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
              Gtk.ResponseType.OK))
         book_chooser_button = Gtk.FileChooserButton.new_with_dialog(
@@ -170,7 +171,7 @@ class HintTab:
         conf.set("egtb_path", egtb_path)
 
         egtb_chooser_dialog = Gtk.FileChooserDialog(
-            _("Select Gaviota TB path"), None,
+            _("Select Gaviota TB path"), mainwindow(),
             Gtk.FileChooserAction.SELECT_FOLDER,
             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
              Gtk.ResponseType.OK))
@@ -200,7 +201,6 @@ class HintTab:
         uistuff.createCombo(widgets["ana_combobox"], data, name="ana_combobox")
         uistuff.createCombo(widgets["inv_ana_combobox"], data, name="inv_ana_combobox")
 
-        @idle_add
         def update_analyzers_store(discoverer):
             data = [(item[0], item[1]) for item in newGameDialog.analyzerItems]
             uistuff.updateCombo(widgets["ana_combobox"], data)
@@ -219,16 +219,16 @@ class HintTab:
 
         def on_analyzer_check_toggled(check):
             self.widgets["analyzers_vbox"].set_sensitive(check.get_active())
-            from pychess.widgets.ionest import game_handler
             from pychess.widgets.gamewidget import widgets
-            if len(game_handler.gamewidgets) != 0:
+            perspective = perspective_manager.get_perspective("games")
+            if len(perspective.gamewidgets) != 0:
                 if check.get_active():
-                    for gmwidg in game_handler.gamewidgets:
-                        gmwidg.gamemodel.restart_analyzer(HINT)
+                    for gmwidg in perspective.gamewidgets:
+                        asyncio.async(gmwidg.gamemodel.restart_analyzer(HINT))
                         if not widgets["hint_mode"].get_active():
                             gmwidg.gamemodel.pause_analyzer(HINT)
                 else:
-                    for gmwidg in game_handler.gamewidgets:
+                    for gmwidg in perspective.gamewidgets:
                         gmwidg.gamemodel.remove_analyzer(HINT)
 
         self.widgets["analyzers_vbox"].set_sensitive(widgets[
@@ -238,15 +238,15 @@ class HintTab:
 
         def on_invanalyzer_check_toggled(check):
             self.widgets["inv_analyzers_vbox"].set_sensitive(check.get_active())
-            from pychess.widgets.ionest import game_handler
-            if len(game_handler.gamewidgets) != 0:
+            perspective = perspective_manager.get_perspective("games")
+            if len(perspective.gamewidgets) != 0:
                 if check.get_active():
-                    for gmwidg in game_handler.gamewidgets:
-                        gmwidg.gamemodel.restart_analyzer(SPY)
+                    for gmwidg in perspective.gamewidgets:
+                        asyncio.async(gmwidg.gamemodel.restart_analyzer(SPY))
                         if not widgets["spy_mode"].get_active():
                             gmwidg.gamemodel.pause_analyzer(SPY)
                 else:
-                    for gmwidg in game_handler.gamewidgets:
+                    for gmwidg in perspective.gamewidgets:
                         gmwidg.gamemodel.remove_analyzer(SPY)
 
         self.widgets["inv_analyzers_vbox"].set_sensitive(widgets[
@@ -363,7 +363,7 @@ class SoundTab:
         # Init open dialog
 
         opendialog = Gtk.FileChooserDialog(
-            _("Open Sound File"), None, Gtk.FileChooserAction.OPEN,
+            _("Open Sound File"), mainwindow(), Gtk.FileChooserAction.OPEN,
             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
              Gtk.ResponseType.ACCEPT))
 
@@ -465,7 +465,7 @@ class PanelTab:
         # Put panels in trees
         self.widgets = widgets
 
-        from pychess.widgets.gamewidget import sidePanels, dockLocation
+        from pychess.perspectives.games import sidePanels, dockLocation
 
         saved_panels = []
         xmlOK = os.path.isfile(dockLocation)
@@ -519,7 +519,7 @@ class PanelTab:
         path = store.get_path(iter)
         panel = store[path][3]
 
-        d = Gtk.MessageDialog(type=Gtk.MessageType.INFO,
+        d = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.INFO,
                               buttons=Gtk.ButtonsType.CLOSE)
         d.set_markup("<big><b>%s</b></big>" % panel.__title__)
         text = panel.__about__ if hasattr(
@@ -542,20 +542,19 @@ class PanelTab:
     def __set_panel_active(self, panel, active):
         name = panel.__name__
 
-        from pychess.widgets.gamewidget import notebooks, docks
         from pychess.widgets.pydock import EAST
 
+        persp = perspective_manager.get_perspective("games")
         if active:
-            leaf = notebooks["board"].get_parent().get_parent()
-            leaf.dock(docks[name][1], EAST, docks[name][0], name)
+            leaf = persp.notebooks["board"].get_parent().get_parent()
+            leaf.dock(persp.docks[name][1], EAST, persp.docks[name][0], name)
         else:
             try:
-                notebooks[name].get_parent().get_parent().undock(notebooks[
-                    name])
+                persp.notebooks[name].get_parent().get_parent().undock(persp.notebooks[name])
             except AttributeError:
                 # A new panel appeared in the panels directory
-                leaf = notebooks["board"].get_parent().get_parent()
-                leaf.dock(docks[name][1], EAST, docks[name][0], name)
+                leaf = persp.notebooks["board"].get_parent().get_parent()
+                leaf.dock(persp.docks[name][1], EAST, persp.docks[name][0], name)
 
     def showit(self):
         from pychess.widgets.gamewidget import showDesignGW
@@ -596,7 +595,7 @@ class ThemeTab:
         conf.set("welcome_image", path)
 
         image_chooser_dialog = Gtk.FileChooserDialog(
-            _("Select background image file"), None, Gtk.FileChooserAction.OPEN,
+            _("Select background image file"), mainwindow(), Gtk.FileChooserAction.OPEN,
             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
              Gtk.ResponseType.OK))
         image_chooser_button = Gtk.FileChooserButton.new_with_dialog(
@@ -629,7 +628,7 @@ class ThemeTab:
 
         # Board Colours
 
-        style_ctxt = widgets["window1"].get_style_context()
+        style_ctxt = widgets["main_window"].get_style_context()
         LIGHT = hexcol(style_ctxt.lookup_color("p_light_color")[1])
         DARK = hexcol(style_ctxt.lookup_color("p_dark_color")[1])
 
@@ -767,8 +766,7 @@ class SaveTab:
             """
 
             checkbox = widgets["autoSave"]
-            widgets["autosave_grid"].set_property("sensitive",
-                                                  checkbox.get_active())
+            widgets["autosave_grid"].set_property("sensitive", checkbox.get_active())
 
         conf.notify_add("autoSave", checkCallBack)
         widgets["autoSave"].set_active(False)
@@ -780,7 +778,7 @@ class SaveTab:
         conf.set("autoSavePath", self.auto_save_path)
 
         auto_save_chooser_dialog = Gtk.FileChooserDialog(
-            _("Select auto save path"), None,
+            _("Select auto save path"), mainwindow(),
             Gtk.FileChooserAction.SELECT_FOLDER,
             (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
              Gtk.ResponseType.OK))
