@@ -1,5 +1,3 @@
-from __future__ import absolute_import
-from __future__ import print_function
 
 from gi.repository import Gtk, GObject
 
@@ -7,7 +5,7 @@ from .__init__ import NORTH, EAST, SOUTH, WEST, CENTER, reprPos
 
 
 class PyDockComposite(Gtk.Alignment):
-    def __init__(self, position):
+    def __init__(self, position, perspective):
         GObject.GObject.__init__(self, xscale=1, yscale=1)
 
         if position == NORTH or position == SOUTH:
@@ -15,6 +13,7 @@ class PyDockComposite(Gtk.Alignment):
         elif position == EAST or position == WEST:
             paned = Gtk.HPaned()
         self.position = position
+        self.perspective = perspective
         self.paned = paned
         self.add(self.paned)
         self.paned.show()
@@ -34,8 +33,8 @@ class PyDockComposite(Gtk.Alignment):
         while not isinstance(parent, PyDockComposite):
             parent = parent.get_parent()
         from .PyDockLeaf import PyDockLeaf
-        leaf = PyDockLeaf(widget, title, id)
-        new = PyDockComposite(position)
+        leaf = PyDockLeaf(widget, title, id, self.perspective)
+        new = PyDockComposite(position, self.perspective)
         parent.changeComponent(self, new)
         new.initChildren(self, leaf)
         return leaf
@@ -76,6 +75,7 @@ class PyDockComposite(Gtk.Alignment):
         new.show()
 
         def cb(widget, allocation):
+            # Set initial position of the divider between the two panes of Gtk.Paned
             if allocation.height != 1:
                 if self.position == NORTH:
                     pos = 0.381966011 * allocation.height
@@ -85,6 +85,17 @@ class PyDockComposite(Gtk.Alignment):
                     pos = 0.381966011 * allocation.width
                 elif self.position == EAST:
                     pos = 0.618033989 * allocation.width
+
+                # print(widget.get_children())
+                if hasattr(widget.get_children()[0], "panels"):
+                    widgt, title, id = widget.get_children()[0].panels[0]
+                    if id == "switcher":
+                        pos = 0.1 * allocation.height
+                    elif id == "filter":
+                        pos = 0.1 * allocation.height
+                    elif id == "gamelist":
+                        pos = 1.4 * allocation.height
+
                 widget.set_position(int(pos + .5))
                 widget.disconnect(conid)
 
