@@ -1,7 +1,5 @@
-from __future__ import absolute_import
-from __future__ import print_function
 
-from gi.repository import Gtk, GObject
+from gi.repository import Gtk
 
 from pychess.System.prefix import addDataPrefix
 
@@ -13,8 +11,9 @@ from .HighlightArea import HighlightArea
 
 
 class PyDockLeaf(TabReceiver):
-    def __init__(self, widget, title, id):
-        TabReceiver.__init__(self)
+    def __init__(self, widget, title, id, perspective):
+        TabReceiver.__init__(self, perspective)
+        self.perspective = perspective
         self.set_no_show_all(True)
 
         def customGetTabLabelText(child):
@@ -26,7 +25,7 @@ class PyDockLeaf(TabReceiver):
 
         self.book = Gtk.Notebook()
         self.book.set_name(id)
-        #self.book.get_tab_label_text = customGetTabLabelText
+
         self.book_cids = [
             self.book.connect("drag-begin", self.__onDragBegin),
             self.book.connect("drag-end", self.__onDragEnd),
@@ -34,11 +33,8 @@ class PyDockLeaf(TabReceiver):
         ]
         self.add(self.book)
         self.book.show()
-        # self.book.props.tab_vborder = 0
-        # self.book.props.tab_hborder = 1
 
         self.highlightArea = HighlightArea(self)
-        # self.put(self.highlightArea, 0, 0)
 
         self.button_cids = []
 
@@ -49,7 +45,7 @@ class PyDockLeaf(TabReceiver):
             addDataPrefix("glade/dock_left.svg"),
             addDataPrefix("glade/dock_center.svg"),
             addDataPrefix("glade/dock_star.svg"))
-        # self.put(self.starButton, 0, 0)
+
         self.button_cids += [
             self.starButton.connect("dropped", self.__onDrop),
             self.starButton.connect("hovered", self.__onHover),
@@ -67,19 +63,20 @@ class PyDockLeaf(TabReceiver):
         self.__add(widget, title, id)
 
     def _del(self):
-        self.highlightArea.disconnect(self.highlightArea.cid)
+        if self.highlightArea.handler_is_connected(self.highlightArea.cid):
+            self.highlightArea.disconnect(self.highlightArea.cid)
 
         for cid in self.button_cids:
-            self.starButton.disconnect(cid)
+            if self.starButton.handler_is_connected(cid):
+                self.starButton.disconnect(cid)
         self.button_cids = []
 
         for cid in self.book_cids:
-            self.book.disconnect(cid)
+            if self.book.handler_is_connected(cid):
+                self.book.disconnect(cid)
 
         self.starButton.myparent = None
         self.highlightArea.myparent = None
-        #self.starButton = None
-        #self.highlightArea = None
 
         TabReceiver._del(self)
 
@@ -91,10 +88,8 @@ class PyDockLeaf(TabReceiver):
         return s + " (" + ", ".join(panels) + ")"
 
     def __add(self, widget, title, id):
-        # widget = BorderBox(widget, top=True)
         self.panels.append((widget, title, id))
         self.book.append_page(widget, title)
-        # self.book.set_tab_label_packing(widget, True, True, Gtk.PACK_START)
         self.book.set_tab_detachable(widget, True)
         self.book.set_tab_reorderable(widget, True)
         widget.show_all()
@@ -111,8 +106,8 @@ class PyDockLeaf(TabReceiver):
             while not isinstance(parent, PyDockComposite):
                 parent = parent.get_parent()
 
-            leaf = PyDockLeaf(widget, title, id)
-            new = PyDockComposite(position)
+            leaf = PyDockLeaf(widget, title, id, self.perspective)
+            new = PyDockComposite(position, self.perspective)
             parent.changeComponent(self, new)
             new.initChildren(self, leaf)
             new.show_all()
@@ -122,6 +117,15 @@ class PyDockLeaf(TabReceiver):
         """ remove the widget from the leaf-notebook
             if this was the only widget, remove this leaf from its owner """
 
+        gtk_version = (Gtk.get_major_version(), Gtk.get_minor_version())
+        if gtk_version >= (3, 16):
+            self.book.detach_tab(widget)
+        else:
+            # To not destroy accidentally our panel widget we need to add a reference to it
+            # https://lazka.github.io/pgi-docs/#Gtk-3.0/classes/Container.html#Gtk.Container.remove
+            widget._ref()
+            self.book.remove(widget)
+
         for i, (widget_, title, id) in enumerate(self.panels):
             if widget_ == widget:
                 break
@@ -129,18 +133,12 @@ class PyDockLeaf(TabReceiver):
             raise KeyError("No %s in %s" % (widget, self))
         del self.panels[i]
 
-        self.book.remove_page(self.book.page_num(widget))
         if self.book.get_n_pages() == 0:
-
-            def cb():
-                parent = self.get_parent()
-                while not isinstance(parent, PyDockComposite):
-                    parent = parent.get_parent()
-                parent.removeComponent(self)
-                self._del()
-            # We need to idle_add this, as the widget won't emit drag-ended, if
-            # it is removed to early
-            GObject.idle_add(cb)
+            parent = self.get_parent()
+            while not isinstance(parent, PyDockComposite):
+                parent = parent.get_parent()
+            parent.removeComponent(self)
+            self._del()
 
         return title, id
 
@@ -219,11 +217,11 @@ class PyDockLeaf(TabReceiver):
         self.highlightArea.hide()
 
     def __onDragBegin(self, widget, context):
-        for instance in self.getInstances():
+        for instance in self.getInstances(self.perspective):
             instance.showArrows()
 
     def __onDragEnd(self, widget, context):
-        for instance in self.getInstances():
+        for instance in self.getInstances(self.perspective):
             instance.hideArrows()
 
     def __onDrop(self, starButton, position, sender):
@@ -231,7 +229,7 @@ class PyDockLeaf(TabReceiver):
 
         # if the undocked leaf was alone, __onDragEnd may not triggered
         # because leaf was removed
-        for instance in self.getInstances():
+        for instance in self.getInstances(self.perspective):
             instance.hideArrows()
 
         if self.dockable:
